# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import numpy
import sys

from PySide6.QtCore import (QEvent, QFile, QIODevice, QPointF, QRectF, QSize,
                            qFatal, qWarning, Qt)
from PySide6.QtGui import (QColor, QFont, QGradient, QImage, QMatrix4x4,
                           QPainter, QPlatformSurfaceEvent, QSurface, QWindow)
from PySide6.QtGui import (QRhi, QRhiBuffer,
                           QRhiDepthStencilClearValue,
                           QRhiGraphicsPipeline, QRhiNullInitParams,
                           QRhiGles2InitParams, QRhiRenderBuffer,
                           QRhiSampler, QRhiShaderResourceBinding,
                           QRhiShaderStage, QRhiTexture,
                           QRhiVertexInputAttribute, QRhiVertexInputBinding,
                           QRhiVertexInputLayout, QRhiViewport,
                           QShader)
from PySide6.support import VoidPtr

if sys.platform == "win32":
    from PySide6.QtGui import QRhiD3D11InitParams, QRhiD3D12InitParams
elif sys.platform == "darwin":
    from PySide6.QtGui import QRhiMetalInitParams


# Y up (note clipSpaceCorrMatrix in m_viewProjection), CCW
VERTEX_DATA = numpy.array([
    0.0, 0.5, 1.0, 0.0, 0.0,
    -0.5, -0.5, 0.0, 1.0, 0.0,
    0.5, -0.5, 0.0, 0.0, 1.0], dtype=numpy.float32)


UBUF_SIZE = 68


def getShader(name):
    f = QFile(name)
    if f.open(QIODevice.OpenModeFlag.ReadOnly):
        result = QShader.fromSerialized(f.readAll())
        f.close()
        return result
    return QShader()


class RhiWindow(QWindow):

    def __init__(self, graphicsApi):
        super().__init__()
        self.m_graphicsApi = QRhi.Implementation.Null
        self.m_initialized = False
        self.m_notExposed = False
        self.m_newlyExposed = False

        self.m_fallbackSurface = None
        self.m_rhi = None
        self.m_sc = None
        self.m_ds = None
        self.m_rp = None
        self.m_hasSwapChain = False
        self.m_viewProjection = QMatrix4x4()

        self.m_graphicsApi = graphicsApi

        if graphicsApi == QRhi.Implementation.OpenGLES2:
            self.setSurfaceType(QSurface.SurfaceType.OpenGLSurface)
        elif graphicsApi == QRhi.Implementation.Vulkan:
            self.setSurfaceType(QSurface.SurfaceType.VulkanSurface)
        elif graphicsApi == QRhi.Implementation.D3D11 or graphicsApi == QRhi.Implementation.D3D12:
            self.setSurfaceType(QSurface.SurfaceType.Direct3DSurface)
        elif graphicsApi == QRhi.Implementation.Metal:
            self.setSurfaceType(QSurface.SurfaceType.MetalSurface)
        elif graphicsApi == QRhi.Implementation.Null:
            pass  # RasterSurface

    def __del__(self):
        # destruction order matters to a certain degree: the fallbackSurface
        # must outlive the rhi, the rhi must outlive all other resources.
        # The resources need no special order when destroying.
        del self.m_rp
        self.m_rp = None
        del self.m_ds
        self.m_ds = None
        del self.m_sc
        self.m_sc = None
        del self.m_rhi
        self.m_rhi = None
        if self.m_fallbackSurface:
            del self.m_fallbackSurface
            self.m_fallbackSurface = None

    def graphicsApiName(self):
        if self.m_graphicsApi == QRhi.Implementation.Null:
            return "Null (no output)"
        if self.m_graphicsApi == QRhi.Implementation.OpenGLES2:
            return "OpenGL"
        if self.m_graphicsApi == QRhi.Implementation.Vulkan:
            return "Vulkan"
        if self.m_graphicsApi == QRhi.Implementation.D3D11:
            return "Direct3D 11"
        if self.m_graphicsApi == QRhi.Implementation.D3D12:
            return "Direct3D 12"
        if self.m_graphicsApi == QRhi.Implementation.Metal:
            return "Metal"
        return ""

    def customInit(self):
        pass

    def customRender(self):
        pass

    def exposeEvent(self, e):
        # initialize and start rendering when the window becomes usable
        # for graphics purposes
        is_exposed = self.isExposed()
        if is_exposed and not self.m_initialized:
            self.init()
            self.resizeSwapChain()
            self.m_initialized = True

        surfaceSize = self.m_sc.surfacePixelSize() if self.m_hasSwapChain else QSize()

        # stop pushing frames when not exposed (or size is 0)
        if ((not is_exposed or (self.m_hasSwapChain and surfaceSize.isEmpty()))
                and self.m_initialized and not self.m_notExposed):
            self.m_notExposed = True

        # Continue when exposed again and the surface has a valid size. Note
        # that surfaceSize can be (0, 0) even though size() reports a valid
        # one, hence trusting surfacePixelSize() and not QWindow.
        if is_exposed and self.m_initialized and self.m_notExposed and not surfaceSize.isEmpty():
            self.m_notExposed = False
            self.m_newlyExposed = True

        # always render a frame on exposeEvent() (when exposed) in order to
        # update immediately on window resize.
        if is_exposed and not surfaceSize.isEmpty():
            self.render()

    def event(self, e):
        if e.type() == QEvent.Type.UpdateRequest:
            self.render()
        elif e.type() == QEvent.Type.PlatformSurface:
            # this is the proper time to tear down the swapchain (while
            # the native window and surface are still around)
            if e.surfaceEventType() == QPlatformSurfaceEvent.SurfaceEventType.SurfaceAboutToBeDestroyed:  # noqa: E501
                self.releaseSwapChain()

        return super().event(e)

    def init(self):
        if self.m_graphicsApi == QRhi.Implementation.Null:
            params = QRhiNullInitParams()
            self.m_rhi = QRhi.create(QRhi.Implementation.Null, params)

        if self.m_graphicsApi == QRhi.Implementation.OpenGLES2:
            self.m_fallbackSurface = QRhiGles2InitParams.newFallbackSurface()
            params = QRhiGles2InitParams()
            params.fallbackSurface = self.m_fallbackSurface
            params.window = self
            self.m_rhi = QRhi.create(QRhi.Implementation.OpenGLES2, params)
        elif self.m_graphicsApi == QRhi.Implementation.D3D11:
            params = QRhiD3D11InitParams()
            # Enable the debug layer, if available. This is optional
            # and should be avoided in production builds.
            params.enableDebugLayer = True
            self.m_rhi = QRhi.create(QRhi.Implementation.D3D11, params)
        elif self.m_graphicsApi == QRhi.Implementation.D3D12:
            params = QRhiD3D12InitParams()
            # Enable the debug layer, if available. This is optional
            # and should be avoided in production builds.
            params.enableDebugLayer = True
            self.m_rhi = QRhi.create(QRhi.Implementation.D3D12, params)
        elif self.m_graphicsApi == QRhi.Implementation.Metal:
            params = QRhiMetalInitParams()
            self.m_rhi = QRhi.create(QRhi.Implementation.Metal, params)

        if not self.m_rhi:
            qFatal("Failed to create RHI backend")

        self.m_sc = self.m_rhi.newSwapChain()
        # no need to set the size here, due to UsedWithSwapChainOnly
        self.m_ds = self.m_rhi.newRenderBuffer(QRhiRenderBuffer.Type.DepthStencil,
                                               QSize(), 1,
                                               QRhiRenderBuffer.Flag.UsedWithSwapChainOnly)
        self.m_sc.setWindow(self)
        self.m_sc.setDepthStencil(self.m_ds)
        self.m_rp = self.m_sc.newCompatibleRenderPassDescriptor()
        self.m_sc.setRenderPassDescriptor(self.m_rp)
        self.customInit()

    def resizeSwapChain(self):
        self.m_hasSwapChain = self.m_sc.createOrResize()  # also handles self.m_ds
        outputSize = self.m_sc.currentPixelSize()
        self.m_viewProjection = self.m_rhi.clipSpaceCorrMatrix()
        r = float(outputSize.width()) / float(outputSize.height())
        self.m_viewProjection.perspective(45.0, r, 0.01, 1000.0)
        self.m_viewProjection.translate(0, 0, -4)

    def releaseSwapChain(self):
        if self.m_hasSwapChain:
            self.m_hasSwapChain = False
            self.m_sc.destroy()

    def render(self):
        if not self.m_hasSwapChain or self.m_notExposed:
            return

        # If the window got resized or newly exposed, resize the swapchain.
        # (the newly-exposed case is not actually required by some platforms,
        # but is here for robustness and portability)
        #
        # This (exposeEvent + the logic here) is the only safe way to perform
        # resize handling. Note the usage of the RHI's surfacePixelSize(), and
        # never QWindow::size(). (the two may or may not be the same under the
        # hood, # depending on the backend and platform)
        if self.m_sc.currentPixelSize() != self.m_sc.surfacePixelSize() or self.m_newlyExposed:
            self.resizeSwapChain()
            if not self.m_hasSwapChain:
                return
            self.m_newlyExposed = False

        result = self.m_rhi.beginFrame(self.m_sc)
        if result == QRhi.FrameOpResult.FrameOpSwapChainOutOfDate:
            self.resizeSwapChain()
            if not self.m_hasSwapChain:
                return
            result = self.m_rhi.beginFrame(self.m_sc)

        if result != QRhi.FrameOpResult.FrameOpSuccess:
            qWarning(f"beginFrame failed with {result}, will retry")
            self.requestUpdate()
            return

        self.customRender()

        self.m_rhi.endFrame(self.m_sc)

        # Always request the next frame via requestUpdate(). On some platforms
        # this is backed by a platform-specific solution, e.g. CVDisplayLink
        # on macOS, which is potentially more efficient than a timer,
        # queued metacalls, etc.
        self.requestUpdate()


class HelloWindow(RhiWindow):

    def __init__(self, graphicsApi):
        super().__init__(graphicsApi)
        self.m_vbuf = None
        self.m_ubuf = None
        self.m_texture = None
        self.m_sampler = None
        self.m_colorTriSrb = None
        self.m_colorPipeline = None
        self.m_fullscreenQuadSrb = None
        self.m_fullscreenQuadPipeline = None
        self.m_initialUpdates = None

        self.m_rotation = 0
        self.m_opacity = 1
        self.m_opacityDir = -1

    def ensureFullscreenTexture(self, pixelSize, u):
        if self.m_texture and self.m_texture.pixelSize() == pixelSize:
            return

        if not self.m_texture:
            self.m_texture = self.m_rhi.newTexture(QRhiTexture.Format.RGBA8, pixelSize)
        else:
            self.m_texture.setPixelSize(pixelSize)
        self.m_texture.create()
        image = QImage(pixelSize, QImage.Format.Format_RGBA8888_Premultiplied)
        with QPainter(image) as painter:
            painter.fillRect(QRectF(QPointF(0, 0), pixelSize),
                             QColor.fromRgbF(0.4, 0.7, 0.0, 1.0))
            painter.setPen(Qt.GlobalColor.transparent)
            painter.setBrush(QGradient(QGradient.Preset.DeepBlue))
            painter.drawRoundedRect(QRectF(QPointF(20, 20), pixelSize - QSize(40, 40)),
                                    16, 16)
            painter.setPen(Qt.GlobalColor.black)
            font = QFont()
            font.setPixelSize(0.05 * min(pixelSize.width(), pixelSize.height()))
            painter.setFont(font)
            name = self.graphicsApiName()
            t = (f"Rendering with QRhi to a resizable QWindow.\nThe 3D API is {name}."
                 "\nUse the command-line options to choose a different API.")
            painter.drawText(QRectF(QPointF(60, 60), pixelSize - QSize(120, 120)), 0, t)

        if self.m_rhi.isYUpInNDC():
            image = image.mirrored()

        u.uploadTexture(self.m_texture, image)

    def customInit(self):
        self.m_initialUpdates = self.m_rhi.nextResourceUpdateBatch()

        vertex_size = 4 * VERTEX_DATA.size
        self.m_vbuf = self.m_rhi.newBuffer(QRhiBuffer.Type.Immutable,
                                           QRhiBuffer.UsageFlag.VertexBuffer,
                                           vertex_size)
        self.m_vbuf.create()
        self.m_initialUpdates.uploadStaticBuffer(self.m_vbuf,
                                                 VoidPtr(VERTEX_DATA.tobytes(), vertex_size))

        self.m_ubuf = self.m_rhi.newBuffer(QRhiBuffer.Type.Dynamic,
                                           QRhiBuffer.UsageFlag.UniformBuffer, UBUF_SIZE)
        self.m_ubuf.create()

        self.ensureFullscreenTexture(self.m_sc.surfacePixelSize(), self.m_initialUpdates)

        self.m_sampler = self.m_rhi.newSampler(QRhiSampler.Filter.Linear,
                                               QRhiSampler.Filter.Linear,
                                               QRhiSampler.Filter.None_,
                                               QRhiSampler.AddressMode.ClampToEdge,
                                               QRhiSampler.AddressMode.ClampToEdge)
        self.m_sampler.create()

        self.m_colorTriSrb = self.m_rhi.newShaderResourceBindings()
        visibility = (QRhiShaderResourceBinding.StageFlag.VertexStage
                      | QRhiShaderResourceBinding.StageFlag.FragmentStage)
        bindings = [
            QRhiShaderResourceBinding.uniformBuffer(0, visibility, self.m_ubuf)
        ]
        self.m_colorTriSrb.setBindings(bindings)
        self.m_colorTriSrb.create()

        self.m_colorPipeline = self.m_rhi.newGraphicsPipeline()
        # Enable depth testing; not quite needed for a simple triangle, but we
        # have a depth-stencil buffer so why not.
        self.m_colorPipeline.setDepthTest(True)
        self.m_colorPipeline.setDepthWrite(True)
        # Blend factors default to One, OneOneMinusSrcAlpha, which is convenient.
        premulAlphaBlend = QRhiGraphicsPipeline.TargetBlend()
        premulAlphaBlend.enable = True
        self.m_colorPipeline.setTargetBlends([premulAlphaBlend])
        stages = [
            QRhiShaderStage(QRhiShaderStage.Type.Vertex, getShader(":/color.vert.qsb")),
            QRhiShaderStage(QRhiShaderStage.Type.Fragment, getShader(":/color.frag.qsb"))
        ]
        self.m_colorPipeline.setShaderStages(stages)
        inputLayout = QRhiVertexInputLayout()
        input_bindings = [QRhiVertexInputBinding(5 * 4)]  # sizeof(float)
        inputLayout.setBindings(input_bindings)
        attributes = [
            QRhiVertexInputAttribute(0, 0, QRhiVertexInputAttribute.Format.Float2, 0),
            # sizeof(float)
            QRhiVertexInputAttribute(0, 1, QRhiVertexInputAttribute.Format.Float3, 2 * 4)]
        inputLayout.setAttributes(attributes)
        self.m_colorPipeline.setVertexInputLayout(inputLayout)
        self.m_colorPipeline.setShaderResourceBindings(self.m_colorTriSrb)
        self.m_colorPipeline.setRenderPassDescriptor(self.m_rp)
        self.m_colorPipeline.create()

        self.m_fullscreenQuadSrb = self.m_rhi.newShaderResourceBindings()
        bindings = [
            QRhiShaderResourceBinding.sampledTexture(0, QRhiShaderResourceBinding.StageFlag.FragmentStage,  # noqa: E501
                                                     self.m_texture, self.m_sampler)
        ]
        self.m_fullscreenQuadSrb.setBindings(bindings)
        self.m_fullscreenQuadSrb.create()

        self.m_fullscreenQuadPipeline = self.m_rhi.newGraphicsPipeline()
        stages = [
            QRhiShaderStage(QRhiShaderStage.Type.Vertex, getShader(":/quad.vert.qsb")),
            QRhiShaderStage(QRhiShaderStage.Type.Fragment, getShader(":/quad.frag.qsb"))
        ]
        self.m_fullscreenQuadPipeline.setShaderStages(stages)
        layout = QRhiVertexInputLayout()
        self.m_fullscreenQuadPipeline.setVertexInputLayout(layout)
        self.m_fullscreenQuadPipeline.setShaderResourceBindings(self.m_fullscreenQuadSrb)
        self.m_fullscreenQuadPipeline.setRenderPassDescriptor(self.m_rp)
        self.m_fullscreenQuadPipeline.create()

    def customRender(self):
        resourceUpdates = self.m_rhi.nextResourceUpdateBatch()

        if self.m_initialUpdates:
            resourceUpdates.merge(self.m_initialUpdates)
            self.m_initialUpdates = None

        self.m_rotation += 1.0
        modelViewProjection = self.m_viewProjection
        modelViewProjection.rotate(self.m_rotation, 0, 1, 0)
        projection = numpy.array(modelViewProjection.data(),
                                 dtype=numpy.float32)
        resourceUpdates.updateDynamicBuffer(self.m_ubuf, 0, 64,
                                            projection.tobytes())

        self.m_opacity += self.m_opacityDir * 0.005
        if self.m_opacity < 0.0 or self.m_opacity > 1.0:
            self.m_opacityDir *= -1
            self.m_opacity = max(0.0, min(1.0, self.m_opacity))

        opacity = numpy.array([self.m_opacity], dtype=numpy.float32)
        resourceUpdates.updateDynamicBuffer(self.m_ubuf, 64, 4,
                                            opacity.tobytes())

        cb = self.m_sc.currentFrameCommandBuffer()
        outputSizeInPixels = self.m_sc.currentPixelSize()

        # (re)create the texture with a size matching the output surface size,
        # when necessary.
        self.ensureFullscreenTexture(outputSizeInPixels, resourceUpdates)

        cv = QRhiDepthStencilClearValue(1.0, 0)
        cb.beginPass(self.m_sc.currentFrameRenderTarget(), Qt.GlobalColor.black,
                     cv, resourceUpdates)

        cb.setGraphicsPipeline(self.m_fullscreenQuadPipeline)
        viewport = QRhiViewport(0, 0, outputSizeInPixels.width(),
                                outputSizeInPixels.height())
        cb.setViewport(viewport)
        cb.setShaderResources()
        cb.draw(3)

        cb.setGraphicsPipeline(self.m_colorPipeline)
        cb.setShaderResources()
        vbufBinding = (self.m_vbuf, 0)
        cb.setVertexInput(0, [vbufBinding])
        cb.draw(3)
        cb.endPass()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtSql import QSqlQueryModel
from PySide6.QtWidgets import QTableView, QApplication

import createdb

if __name__ == "__main__":
    app = QApplication()
    createdb.init_db()

    model = QSqlQueryModel()
    model.setQuery("select * from books")

    table_view = QTableView()
    table_view.setModel(model)
    table_view.resize(800, 600)
    table_view.show()
    sys.exit(app.exec())



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'mainwindow.ui'
##
## Created by: Qt User Interface Compiler version 6.8.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QAbstractItemView, QApplication, QGridLayout, QGroupBox,
    QHBoxLayout, QHeaderView, QLabel, QMainWindow,
    QMenu, QMenuBar, QSizePolicy, QSpacerItem,
    QStatusBar, QToolBar, QVBoxLayout, QWidget)

from receivedframesview import ReceivedFramesView
from sendframebox import SendFrameBox
import rc_can

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(634, 527)
        self.actionConnect = QAction(MainWindow)
        self.actionConnect.setObjectName(u"actionConnect")
        icon = QIcon()
        icon.addFile(u":/images/connect.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.actionConnect.setIcon(icon)
        self.actionDisconnect = QAction(MainWindow)
        self.actionDisconnect.setObjectName(u"actionDisconnect")
        icon1 = QIcon()
        icon1.addFile(u":/images/disconnect.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.actionDisconnect.setIcon(icon1)
        self.actionQuit = QAction(MainWindow)
        self.actionQuit.setObjectName(u"actionQuit")
        icon2 = QIcon()
        icon2.addFile(u":/images/application-exit.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.actionQuit.setIcon(icon2)
        self.actionAboutQt = QAction(MainWindow)
        self.actionAboutQt.setObjectName(u"actionAboutQt")
        self.actionClearLog = QAction(MainWindow)
        self.actionClearLog.setObjectName(u"actionClearLog")
        icon3 = QIcon()
        icon3.addFile(u":/images/clear.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.actionClearLog.setIcon(icon3)
        self.actionPluginDocumentation = QAction(MainWindow)
        self.actionPluginDocumentation.setObjectName(u"actionPluginDocumentation")
        self.actionResetController = QAction(MainWindow)
        self.actionResetController.setObjectName(u"actionResetController")
        self.actionDeviceInformation = QAction(MainWindow)
        self.actionDeviceInformation.setObjectName(u"actionDeviceInformation")
        self.centralWidget = QWidget(MainWindow)
        self.centralWidget.setObjectName(u"centralWidget")
        self.verticalLayout = QVBoxLayout(self.centralWidget)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.sendFrameBox = SendFrameBox(self.centralWidget)
        self.sendFrameBox.setObjectName(u"sendFrameBox")

        self.verticalLayout.addWidget(self.sendFrameBox)

        self.receivedMessagesBox = QGroupBox(self.centralWidget)
        self.receivedMessagesBox.setObjectName(u"receivedMessagesBox")
        self.gridLayout = QGridLayout(self.receivedMessagesBox)
        self.gridLayout.setSpacing(6)
        self.gridLayout.setContentsMargins(11, 11, 11, 11)
        self.gridLayout.setObjectName(u"gridLayout")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setSpacing(6)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.receivedFramesView = ReceivedFramesView(self.receivedMessagesBox)
        self.receivedFramesView.setObjectName(u"receivedFramesView")
        self.receivedFramesView.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
        self.receivedFramesView.setProperty(u"showDropIndicator", False)
        self.receivedFramesView.setDragDropOverwriteMode(False)
        self.receivedFramesView.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)

        self.verticalLayout_2.addWidget(self.receivedFramesView)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(6)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.busStatus = QLabel(self.receivedMessagesBox)
        self.busStatus.setObjectName(u"busStatus")

        self.horizontalLayout.addWidget(self.busStatus)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)


        self.verticalLayout_2.addLayout(self.horizontalLayout)


        self.gridLayout.addLayout(self.verticalLayout_2, 0, 0, 1, 1)


        self.verticalLayout.addWidget(self.receivedMessagesBox)

        MainWindow.setCentralWidget(self.centralWidget)
        self.menuBar = QMenuBar(MainWindow)
        self.menuBar.setObjectName(u"menuBar")
        self.menuBar.setGeometry(QRect(0, 0, 634, 26))
        self.menuCalls = QMenu(self.menuBar)
        self.menuCalls.setObjectName(u"menuCalls")
        self.menuHelp = QMenu(self.menuBar)
        self.menuHelp.setObjectName(u"menuHelp")
        MainWindow.setMenuBar(self.menuBar)
        self.mainToolBar = QToolBar(MainWindow)
        self.mainToolBar.setObjectName(u"mainToolBar")
        MainWindow.addToolBar(Qt.ToolBarArea.TopToolBarArea, self.mainToolBar)
        self.statusBar = QStatusBar(MainWindow)
        self.statusBar.setObjectName(u"statusBar")
        MainWindow.setStatusBar(self.statusBar)

        self.menuBar.addAction(self.menuCalls.menuAction())
        self.menuBar.addAction(self.menuHelp.menuAction())
        self.menuCalls.addAction(self.actionConnect)
        self.menuCalls.addAction(self.actionDisconnect)
        self.menuCalls.addAction(self.actionDeviceInformation)
        self.menuCalls.addSeparator()
        self.menuCalls.addAction(self.actionResetController)
        self.menuCalls.addSeparator()
        self.menuCalls.addAction(self.actionClearLog)
        self.menuCalls.addSeparator()
        self.menuCalls.addAction(self.actionQuit)
        self.menuHelp.addAction(self.actionPluginDocumentation)
        self.menuHelp.addAction(self.actionAboutQt)
        self.mainToolBar.addAction(self.actionConnect)
        self.mainToolBar.addAction(self.actionDisconnect)
        self.mainToolBar.addSeparator()
        self.mainToolBar.addAction(self.actionClearLog)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"CAN Example", None))
        self.actionConnect.setText(QCoreApplication.translate("MainWindow", u"&Connect", None))
        self.actionDisconnect.setText(QCoreApplication.translate("MainWindow", u"&Disconnect", None))
        self.actionQuit.setText(QCoreApplication.translate("MainWindow", u"&Quit", None))
        self.actionAboutQt.setText(QCoreApplication.translate("MainWindow", u"&About Qt", None))
        self.actionClearLog.setText(QCoreApplication.translate("MainWindow", u"Clear &Log", None))
        self.actionPluginDocumentation.setText(QCoreApplication.translate("MainWindow", u"Plugin Documentation", None))
#if QT_CONFIG(tooltip)
        self.actionPluginDocumentation.setToolTip(QCoreApplication.translate("MainWindow", u"Open plugin documentation in Webbrowser", None))
#endif // QT_CONFIG(tooltip)
        self.actionResetController.setText(QCoreApplication.translate("MainWindow", u"&Reset CAN Controller", None))
#if QT_CONFIG(tooltip)
        self.actionResetController.setToolTip(QCoreApplication.translate("MainWindow", u"Reset CAN Controller", None))
#endif // QT_CONFIG(tooltip)
        self.actionDeviceInformation.setText(QCoreApplication.translate("MainWindow", u"Device &Information...", None))
        self.sendFrameBox.setTitle(QCoreApplication.translate("MainWindow", u"Send CAN frame", None))
        self.receivedMessagesBox.setTitle(QCoreApplication.translate("MainWindow", u"Received CAN messages", None))
        self.busStatus.setText("")
        self.menuCalls.setTitle(QCoreApplication.translate("MainWindow", u"&Calls", None))
        self.menuHelp.setTitle(QCoreApplication.translate("MainWindow", u"&Help", None))
    # retranslateUi




# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the corelib/threads/mandelbrot example from Qt v5.x, originating from PyQt"""

from argparse import ArgumentParser, RawTextHelpFormatter
import sys

from PySide6.QtCore import (Signal, QMutex, QElapsedTimer, QMutexLocker,
                            QPoint, QPointF, QRectF, QSize, Qt, QThread,
                            QWaitCondition, Slot)
from PySide6.QtGui import QColor, QImage, QPainter, QPixmap, qRgb
from PySide6.QtWidgets import QApplication, QWidget


DEFAULT_CENTER_X = -0.647011
DEFAULT_CENTER_Y = -0.0395159
DEFAULT_SCALE = 0.00403897

ZOOM_IN_FACTOR = 0.8
ZOOM_OUT_FACTOR = 1 / ZOOM_IN_FACTOR
SCROLL_STEP = 20


NUM_PASSES = 8


INFO_KEY = 'info'


HELP = ("Use mouse wheel or the '+' and '-' keys to zoom. Press and "
        "hold left mouse button to scroll.")


class RenderThread(QThread):
    colormap_size = 512

    rendered_image = Signal(QImage, float)

    def __init__(self, parent=None):
        super().__init__(parent)

        self.mutex = QMutex()
        self.condition = QWaitCondition()
        self._center_x = 0.0
        self._center_y = 0.0
        self._scale_factor = 0.0
        self._result_size = QSize()
        self.colormap = []

        self.restart = False
        self.abort = False

        for i in range(RenderThread.colormap_size):
            self.colormap.append(
                self.rgb_from_wave_length(380.0 + (i * 400.0 / RenderThread.colormap_size)))

    def stop(self):
        self.mutex.lock()
        self.abort = True
        self.condition.wakeOne()
        self.mutex.unlock()

        self.wait(2000)

    def render(self, centerX, centerY, scale_factor, resultSize):
        with QMutexLocker(self.mutex):
            self._center_x = centerX
            self._center_y = centerY
            self._scale_factor = scale_factor
            self._result_size = resultSize

            if not self.isRunning():
                self.start(QThread.Priority.LowPriority)
            else:
                self.restart = True
                self.condition.wakeOne()

    def run(self):
        timer = QElapsedTimer()

        while True:
            self.mutex.lock()
            resultSize = self._result_size
            scale_factor = self._scale_factor
            centerX = self._center_x
            centerY = self._center_y
            self.mutex.unlock()

            half_width = resultSize.width() // 2
            half_height = resultSize.height() // 2
            image = QImage(resultSize, QImage.Format.Format_RGB32)

            curpass = 0

            while curpass < NUM_PASSES:
                timer.restart()
                max_iterations = (1 << (2 * curpass + 6)) + 32
                LIMIT = 4
                all_black = True

                for y in range(-half_height, half_height):
                    if self.restart:
                        break
                    if self.abort:
                        return

                    ay = 1j * (centerY + (y * scale_factor))

                    for x in range(-half_width, half_width):
                        c0 = centerX + (x * scale_factor) + ay
                        c = c0
                        num_iterations = 0

                        while num_iterations < max_iterations:
                            num_iterations += 1
                            c = c * c + c0
                            if abs(c) >= LIMIT:
                                break
                            num_iterations += 1
                            c = c * c + c0
                            if abs(c) >= LIMIT:
                                break
                            num_iterations += 1
                            c = c * c + c0
                            if abs(c) >= LIMIT:
                                break
                            num_iterations += 1
                            c = c * c + c0
                            if abs(c) >= LIMIT:
                                break

                        if num_iterations < max_iterations:
                            image.setPixel(x + half_width, y + half_height,
                                           self.colormap[
                                               num_iterations % RenderThread.colormap_size])
                            all_black = False
                        else:
                            image.setPixel(x + half_width, y + half_height, qRgb(0, 0, 0))

                if all_black and curpass == 0:
                    curpass = 4
                else:
                    if not self.restart:
                        elapsed = timer.elapsed()
                        unit = 'ms'
                        if elapsed > 2000:
                            elapsed /= 1000
                            unit = 's'
                        text = (f"Pass {curpass + 1}/{NUM_PASSES}, "
                                f"max iterations: {max_iterations}, time: {elapsed}{unit}")
                        image.setText(INFO_KEY, text)
                        self.rendered_image.emit(image, scale_factor)
                    curpass += 1

            self.mutex.lock()
            if not self.restart:
                self.condition.wait(self.mutex)
            self.restart = False
            self.mutex.unlock()

    def rgb_from_wave_length(self, wave):
        r = 0.0
        g = 0.0
        b = 0.0

        if wave >= 380.0 and wave <= 440.0:
            r = -1.0 * (wave - 440.0) / (440.0 - 380.0)
            b = 1.0
        elif wave >= 440.0 and wave <= 490.0:
            g = (wave - 440.0) / (490.0 - 440.0)
            b = 1.0
        elif wave >= 490.0 and wave <= 510.0:
            g = 1.0
            b = -1.0 * (wave - 510.0) / (510.0 - 490.0)
        elif wave >= 510.0 and wave <= 580.0:
            r = (wave - 510.0) / (580.0 - 510.0)
            g = 1.0
        elif wave >= 580.0 and wave <= 645.0:
            r = 1.0
            g = -1.0 * (wave - 645.0) / (645.0 - 580.0)
        elif wave >= 645.0 and wave <= 780.0:
            r = 1.0

        s = 1.0
        if wave > 700.0:
            s = 0.3 + 0.7 * (780.0 - wave) / (780.0 - 700.0)
        elif wave < 420.0:
            s = 0.3 + 0.7 * (wave - 380.0) / (420.0 - 380.0)

        r = pow(r * s, 0.8)
        g = pow(g * s, 0.8)
        b = pow(b * s, 0.8)

        return qRgb(int(r * 255), int(g * 255), int(b * 255))


class MandelbrotWidget(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.thread = RenderThread()
        self.pixmap = QPixmap()
        self._pixmap_offset = QPointF()
        self._last_drag_pos = QPointF()

        self._center_x = DEFAULT_CENTER_X
        self._center_y = DEFAULT_CENTER_Y
        self._pixmap_scale = DEFAULT_SCALE
        self._cur_scale = DEFAULT_SCALE

        self.thread.rendered_image.connect(self.update_pixmap)

        self.setWindowTitle("Mandelbrot")
        self.setCursor(Qt.CursorShape.CrossCursor)
        self._info = ''

    def paintEvent(self, event):
        with QPainter(self) as painter:
            painter.fillRect(self.rect(), Qt.GlobalColor.black)

            if self.pixmap.isNull():
                painter.setPen(Qt.GlobalColor.white)
                painter.drawText(self.rect(), Qt.AlignmentFlag.AlignCenter,
                                 "Rendering initial image, please wait...")
                return

            if self._cur_scale == self._pixmap_scale:
                painter.drawPixmap(self._pixmap_offset, self.pixmap)
            else:
                scale_factor = self._pixmap_scale / self._cur_scale
                new_width = int(self.pixmap.width() * scale_factor)
                new_height = int(self.pixmap.height() * scale_factor)
                new_x = self._pixmap_offset.x() + (self.pixmap.width() - new_width) / 2
                new_y = self._pixmap_offset.y() + (self.pixmap.height() - new_height) / 2

                painter.save()
                painter.translate(new_x, new_y)
                painter.scale(scale_factor, scale_factor)
                exposed, _ = painter.transform().inverted()
                exposed = exposed.mapRect(self.rect()).adjusted(-1, -1, 1, 1)
                painter.drawPixmap(exposed, self.pixmap, exposed)
                painter.restore()

            text = HELP
            if self._info:
                text += ' ' + self._info
            metrics = painter.fontMetrics()
            text_width = metrics.horizontalAdvance(text)

            painter.setPen(Qt.PenStyle.NoPen)
            painter.setBrush(QColor(0, 0, 0, 127))
            box = QRectF((self.width() - text_width) / 2 - 5, 0,
                         text_width + 10, metrics.lineSpacing() + 5)
            painter.drawRect(box)
            painter.setPen(Qt.GlobalColor.white)
            pos = QPointF((self.width() - text_width) / 2,
                          metrics.leading() + metrics.ascent())
            painter.drawText(pos, text)

    def resizeEvent(self, event):
        self.thread.render(self._center_x, self._center_y, self._cur_scale, self.size())

    def keyPressEvent(self, event):
        if event.key() == Qt.Key.Key_Plus:
            self.zoom(ZOOM_IN_FACTOR)
        elif event.key() == Qt.Key.Key_Minus:
            self.zoom(ZOOM_OUT_FACTOR)
        elif event.key() == Qt.Key.Key_Left:
            self.scroll(-SCROLL_STEP, 0)
        elif event.key() == Qt.Key.Key_Right:
            self.scroll(+SCROLL_STEP, 0)
        elif event.key() == Qt.Key.Key_Down:
            self.scroll(0, -SCROLL_STEP)
        elif event.key() == Qt.Key.Key_Up:
            self.scroll(0, +SCROLL_STEP)
        elif event.key() == Qt.Key.Key_Q:
            self.close()
        else:
            super(MandelbrotWidget, self).keyPressEvent(event)

    def wheelEvent(self, event):
        num_degrees = event.angleDelta().y() / 8
        num_steps = num_degrees / 15.0
        self.zoom(pow(ZOOM_IN_FACTOR, num_steps))

    def mousePressEvent(self, event):
        if event.buttons() == Qt.MouseButton.LeftButton:
            self._last_drag_pos = event.position()

    def mouseMoveEvent(self, event):
        if event.buttons() & Qt.MouseButton.LeftButton:
            pos = event.position()
            self._pixmap_offset += pos - self._last_drag_pos
            self._last_drag_pos = pos
            self.update()

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.MouseButton.LeftButton:
            pos = event.position()
            self._pixmap_offset += pos - self._last_drag_pos
            self._last_drag_pos = QPointF()

            delta_x = (self.width() - self.pixmap.width()) / 2 - self._pixmap_offset.x()
            delta_y = (self.height() - self.pixmap.height()) / 2 - self._pixmap_offset.y()
            self.scroll(delta_x, delta_y)

    @Slot(QImage, float)
    def update_pixmap(self, image, scale_factor):
        if not self._last_drag_pos.isNull():
            return

        self._info = image.text(INFO_KEY)
        self.pixmap = QPixmap.fromImage(image)
        self._pixmap_offset = QPointF()
        self._last_drag_position = QPointF()
        self._pixmap_scale = scale_factor
        self.update()

    def zoom(self, zoomFactor):
        self._cur_scale *= zoomFactor
        self.update()
        self.thread.render(self._center_x, self._center_y, self._cur_scale, self.size())

    def scroll(self, deltaX, deltaY):
        self._center_x += deltaX * self._cur_scale
        self._center_y += deltaY * self._cur_scale
        self.update()
        self.thread.render(self._center_x, self._center_y, self._cur_scale, self.size())


if __name__ == '__main__':
    parser = ArgumentParser(description='Qt Mandelbrot Example',
                            formatter_class=RawTextHelpFormatter)
    parser.add_argument('--passes', '-p', type=int, help='Number of passes (1-8)')
    options = parser.parse_args()
    if options.passes:
        NUM_PASSES = int(options.passes)
        if NUM_PASSES < 1 or NUM_PASSES > 8:
            print(f'Invalid value: {options.passes}')
            sys.exit(-1)

    app = QApplication(sys.argv)
    widget = MandelbrotWidget()
    geometry = widget.screen().availableGeometry()
    widget.resize((2 * geometry.size()) / 3)
    pos = (geometry.size() - widget.size()) / 2
    widget.move(geometry.topLeft() + QPoint(pos.width(), pos.height()))

    widget.show()
    r = app.exec()
    widget.thread.stop()
    sys.exit(r)



# Copyright (C) 2017 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com, author Milian Wolff <milian.wolff@kdab.com>
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtWebChannel import QWebChannelAbstractTransport
from PySide6.QtCore import QByteArray, QJsonDocument, Slot


class WebSocketTransport(QWebChannelAbstractTransport):
    """QWebChannelAbstractSocket implementation using a QWebSocket internally.

        The transport delegates all messages received over the QWebSocket over
        its textMessageReceived signal. Analogously, all calls to
        sendTextMessage will be sent over the QWebSocket to the remote client.
    """

    def __init__(self, socket):
        """Construct the transport object and wrap the given socket.
           The socket is also set as the parent of the transport object."""
        super().__init__(socket)
        self._socket = socket
        self._socket.textMessageReceived.connect(self.text_message_received)
        self._socket.disconnected.connect(self._disconnected)

    def __del__(self):
        """Destroys the WebSocketTransport."""
        self._socket.deleteLater()

    def _disconnected(self):
        self.deleteLater()

    def sendMessage(self, message):
        """Serialize the JSON message and send it as a text message via the
           WebSocket to the client."""
        doc = QJsonDocument(message)
        json_message = str(doc.toJson(QJsonDocument.Compact), "utf-8")
        self._socket.sendTextMessage(json_message)

    @Slot(str)
    def text_message_received(self, message_data_in):
        """Deserialize the stringified JSON messageData and emit
           messageReceived."""
        message_data = QByteArray(bytes(message_data_in, encoding='utf8'))
        message = QJsonDocument.fromJson(message_data)
        if message.isNull():
            print("Failed to parse text message as JSON object:", message_data)
            return
        if not message.isObject():
            print("Received JSON message that is not an object: ", message_data)
            return
        self.messageReceived.emit(message.object(), self)



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QDate, QDateTime, QTime, QUrl
from PySide6.QtCore import QLine, QPoint, QRect, QSize


URL = "https://qt.io/"


class HashTest(unittest.TestCase):
    def testInsert(self):
        myHash = {}
        qdate = QDate.currentDate()
        qdatetime = QDateTime.currentDateTime()
        qtime = QTime.currentTime()
        qurl = QUrl(URL)
        self.assertTrue(qurl.isValid())
        qpoint = QPoint(12, 42)

        myHash[qdate] = "QDate"
        myHash[qdatetime] = "QDateTime"
        myHash[qtime] = "QTime"
        myHash[qurl] = "QUrl"
        myHash[qpoint] = "QPoint"

        self.assertEqual(myHash[qdate], "QDate")
        self.assertEqual(myHash[qdatetime], "QDateTime")
        self.assertEqual(myHash[qtime], "QTime")
        self.assertEqual(myHash[qurl], "QUrl")
        self.assertEqual(myHash[qpoint], "QPoint")

    def testQPointHash(self):
        p1 = QPoint(12, 34)
        p2 = QPoint(12, 34)
        self.assertFalse(p1 is p2)
        self.assertEqual(p1, p2)
        self.assertEqual(hash(p1), hash(p2))

    def testQSizeHash(self):
        s1 = QSize(12, 34)
        s2 = QSize(12, 34)
        self.assertFalse(s1 is s2)
        self.assertEqual(s1, s2)
        self.assertEqual(hash(s1), hash(s2))

    def testQRectHash(self):
        r1 = QRect(12, 34, 56, 78)
        r2 = QRect(12, 34, 56, 78)
        self.assertFalse(r1 is r2)
        self.assertEqual(r1, r2)
        self.assertEqual(hash(r1), hash(r2))

    def testQLineHash(self):
        l1 = QLine(12, 34, 56, 78)
        l2 = QLine(12, 34, 56, 78)
        self.assertFalse(l1 is l2)
        self.assertEqual(l1, l2)
        self.assertEqual(hash(l1), hash(l2))

    def testQTimeHash(self):
        t1 = QTime(5, 5, 5)
        t2 = QTime(5, 5, 5)
        self.assertFalse(t1 is t2)
        self.assertEqual(t1, t2)
        self.assertEqual(hash(t1), hash(t2))

    def testQDateHash(self):
        d1 = QDate(1968, 3, 9)
        d2 = QDate(1968, 3, 9)
        self.assertFalse(d1 is d2)
        self.assertEqual(d1, d2)
        self.assertEqual(hash(d1), hash(d2))

    def testQDateTimeHash(self):
        d1 = QDateTime(QDate(1968, 3, 9), QTime(5, 5, 5))
        d2 = QDateTime(QDate(1968, 3, 9), QTime(5, 5, 5))
        self.assertFalse(d1 is d2)
        self.assertEqual(d1, d2)
        self.assertEqual(hash(d1), hash(d2))

    def testQUrlHash(self):
        u1 = QUrl(URL)
        u2 = QUrl(URL)
        self.assertFalse(u1 is u2)
        self.assertEqual(u1, u2)
        self.assertEqual(hash(u1), hash(u2))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import ObjectType


class NewStyle:
    def name(self):
        return "NewStyle"


def defineNewStyle():
    class MyObjectNew(ObjectType, NewStyle):
        pass


class ObjectTypeTest(unittest.TestCase):
    '''Test cases to avoid declaring Shiboken classes with multiple inheritance
       from old style classes.'''

    def testObjectTypeNewStype(self):
        defineNewStyle()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for <add-function> with const char* as argument'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Color, Pen, SampleNamespace


class TestPen(unittest.TestCase):
    '''Simple test case for Pen.'''

    def testPenWithEmptyConstructor(self):
        pen = Pen()
        self.assertEqual(pen.ctorType(), Pen.EmptyCtor)

    def testPenWithEnumConstructor(self):
        pen = Pen(SampleNamespace.Option.RandomNumber)
        self.assertEqual(pen.ctorType(), Pen.EnumCtor)

    def testPenWithColorConstructor(self):
        pen = Pen(Color())
        self.assertEqual(pen.ctorType(), Pen.ColorCtor)

    def testPenWithCopyConstructor(self):
        pen = Pen(Pen())
        self.assertEqual(pen.ctorType(), Pen.CopyCtor)

    def testPenWithIntConvertedToColor(self):
        pen = Pen(1)
        self.assertEqual(pen.ctorType(), Pen.ColorCtor)
        pen.drawLine(0, 0, 5, 5)

    def testPenRenderHintsProperty(self):
        """Exercise the generated property setter and getters, checking
           against the C++ getter/setter functions."""
        pen = Pen(1)
        self.assertEqual(pen.getRenderHints(), Pen.RenderHints.None_)
        self.assertEqual(pen.renderHints, Pen.RenderHints.None_)
        pen.renderHints = Pen.RenderHints.TextAntialiasing
        self.assertEqual(pen.getRenderHints(), Pen.RenderHints.TextAntialiasing)
        self.assertEqual(pen.renderHints, Pen.RenderHints.TextAntialiasing)
        pen.setRenderHints(Pen.RenderHints.Antialiasing)
        self.assertEqual(pen.getRenderHints(), Pen.RenderHints.Antialiasing)
        self.assertEqual(pen.renderHints, Pen.RenderHints.Antialiasing)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import http.server as BaseHTTPServer
import os
import random
import select
import sys
import socketserver as SocketServer
import threading

sys.path.append(os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "util"))


class TestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
    DATA = b"PySide Server"
    allow_reuse_address = True

    def do_GET(self):
        self.send_head()
        self.wfile.write(TestHandler.DATA)

    def do_HEAD(self):
        self.send_head()

    def send_head(self):
        self.send_response(200)
        self.send_header("Content-type", "text/plain")
        self.send_header("Content-Length", str(len(TestHandler.DATA)))
        self.end_headers()


class TestSecureHandler(BaseHTTPServer.BaseHTTPRequestHandler):
    DATA = "PySide"
    allow_reuse_address = True

    def do_GET(self):
        self.send_head()
        self.wfile.write(TestHandler.DATA)

    def do_HEAD(self):
        self.send_head()

    def send_head(self):
        try:
            handler = self.marshall_handler()
            handler.do_request(self)
        except:  # noqa: E722
            self.send_response(401)
            self.send_header("WWW-Authenticate", "Basic realm='Secure Area'")
            self.send_header("Content-type", "text/plain")
            self.send_header("Content-Length", str(len(TestHandler.DATA)))
            self.end_headers()

# Workaround for the missing shutdown method in python2.5


class CompatTCPServer(SocketServer.TCPServer):
    def __init__(self, server_address, RequestHandlerClass):
        SocketServer.TCPServer.__init__(self, server_address, RequestHandlerClass)

        self.isPy25 = sys.version_info[0] == 2 and sys.version_info[1] == 5
        if self.isPy25:
            self.__is_shut_down = threading.Event()
            self.__serving = False

    def serve_forever(self, poll_interval=0.5):
        """Handle one request at a time until shutdown.

        Polls for shutdown every poll_interval seconds. Ignores
        self.timeout. If you need to do periodic tasks, do them in
        another thread.
        """
        if self.isPy25:
            self.__serving = True
            self.__is_shut_down.clear()
            while self.__serving:
                # XXX: Consider using another file descriptor or
                # connecting to the socket to wake this up instead of
                # polling. Polling reduces our responsiveness to a
                # shutdown request and wastes cpu at all other times.
                r, w, e = select.select([self], [], [], poll_interval)
                if r:
                    self.py25_handle_request_noblock()
            self.__is_shut_down.set()
        else:
            SocketServer.TCPServer.serve_forever(self, poll_interval)

    def py25_handle_request_noblock(self):
        """Handle one request, without blocking.

        I assume that select.select has returned that the socket is
        readable before this function was called, so there should be
        no risk of blocking in get_request().
        """
        if self.isPy25:
            try:
                request, client_address = self.get_request()
            except socket.error:  # noqa: F821
                return
            if self.verify_request(request, client_address):
                try:
                    self.process_request(request, client_address)
                except:  # noqa: E722
                    self.handle_error(request, client_address)
                    self.close_request(request)

    def shutdown(self):
        """Stops the serve_forever loop.

        Blocks until the loop has finished. This must be called while
        serve_forever() is running in another thread, or it will
        deadlock.
        """
        if self.isPy25:
            self.__serving = False
            if not self.__is_shut_down:
                self.__is_shut_down.wait()
        else:
            SocketServer.TCPServer.shutdown(self)


class TestServer(threading.Thread):

    def __init__(self, secure=False):
        threading.Thread.__init__(self)

        self._port = int(os.getenv("PYSIDE_TESTSERVER_PORT") or 12321)
        self.keep_running = True

        if secure:
            handle = TestSecureHandler
        else:
            handle = TestHandler

        while True:
            try:
                self.httpd = CompatTCPServer(('', self._port), handle)
                break
            except:  # noqa: E722
                self._port = self._port + random.randint(1, 100)

    def port(self):
        return self._port

    def run(self):
        self.httpd.serve_forever()

    def shutdown(self):
        self.httpd.shutdown()
        self.join()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the networkauth redditclient example from Qt v6.x"""

from argparse import ArgumentParser, RawTextHelpFormatter
import sys

from PySide6.QtWidgets import QApplication, QListView

from redditmodel import RedditModel


if __name__ == '__main__':
    parser = ArgumentParser(description='Qt Reddit client example',
                            formatter_class=RawTextHelpFormatter)
    parser.add_argument('--client', '-i', type=str, help='Client id')
    options = parser.parse_args()
    if not options.client:
        print('Specify a client id', file=sys.stderr)
        sys.exit(-1)

    app = QApplication(sys.argv)
    view = QListView()
    model = RedditModel(options.client)
    view.setModel(model)
    view.show()
    sys.exit(app.exec())



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
from dataclasses import dataclass
from PySide6.QtCore import (QAbstractListModel, QByteArray,
                            QUrlQuery, Property, Signal, Slot, Qt)
from PySide6.QtQml import QmlAnonymous, QmlElement

from abstractresource import AbstractResource


QML_IMPORT_NAME = "ColorPalette"
QML_IMPORT_MAJOR_VERSION = 1


totalPagesField = "total_pages"
currentPageField = "page"


@dataclass
class ColorUser:
    id: int
    email: str
    avatar: str  # URL


@QmlElement
class ColorUserModel (QAbstractListModel):
    IdRole = Qt.ItemDataRole.UserRole + 1
    EmailRole = Qt.ItemDataRole.UserRole + 2
    AvatarRole = Qt.ItemDataRole.UserRole + 3

    def __init__(self, parent=None):
        super().__init__(parent)
        self._users = []

    def clear(self):
        self.set_data([])

    def set_data(self, json_list):
        if not self._users and not json_list:
            return
        self.beginResetModel()
        self._users.clear()
        for e in json_list:
            self._users.append(ColorUser(int(e["id"]), e["email"], e["avatar"]))
        self.endResetModel()

    def roleNames(self):
        roles = {
            ColorUserModel.IdRole: QByteArray(b'id'),
            ColorUserModel.EmailRole: QByteArray(b'email'),
            ColorUserModel.AvatarRole: QByteArray(b'avatar')
        }
        return roles

    def rowCount(self, index):
        return len(self._users)

    def data(self, index, role):
        if index.isValid():
            d = self._users[index.row()]
            if role == ColorUserModel.IdRole:
                return d.id
            if role == ColorUserModel.EmailRole:
                return d.email
            if role == ColorUserModel.AvatarRole:
                return d.avatar
        return None

    def avatarForEmail(self, email):
        for e in self._users:
            if e.email == email:
                return e.avatar
        return ""


@dataclass
class Color:
    id: int
    color: str
    name: str
    pantone_value: str


@QmlElement
class ColorModel (QAbstractListModel):
    IdRole = Qt.ItemDataRole.UserRole + 1
    ColorRole = Qt.ItemDataRole.UserRole + 2
    NameRole = Qt.ItemDataRole.UserRole + 3
    PantoneValueRole = Qt.ItemDataRole.UserRole + 4

    def __init__(self, parent=None):
        super().__init__(parent)
        self._colors = []

    def clear(self):
        self.set_data([])

    def set_data(self, json_list):
        if not self._colors and not json_list:
            return
        self.beginResetModel()
        self._colors.clear()
        for e in json_list:
            self._colors.append(Color(int(e["id"]), e["color"],
                                      e["name"], e["pantone_value"]))
        self.endResetModel()

    def roleNames(self):
        roles = {
            ColorModel.IdRole: QByteArray(b'color_id'),
            ColorModel.ColorRole: QByteArray(b'color'),
            ColorModel.NameRole: QByteArray(b'name'),
            ColorModel.PantoneValueRole: QByteArray(b'pantone_value')
        }
        return roles

    def rowCount(self, index):
        return len(self._colors)

    def data(self, index, role):
        if index.isValid():
            d = self._colors[index.row()]
            if role == ColorModel.IdRole:
                return d.id
            if role == ColorModel.ColorRole:
                return d.color
            if role == ColorModel.NameRole:
                return d.name
            if role == ColorModel.PantoneValueRole:
                return d.pantone_value
        return None


@QmlAnonymous
class PaginatedResource(AbstractResource):
    """This class manages a simple paginated Crud resource,
       where the resource is a paginated list of JSON items."""

    dataUpdated = Signal()
    pageUpdated = Signal()
    pagesUpdated = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        # The total number of pages as reported by the server responses
        self.m_pages = 0
        # The default page we request if the user hasn't set otherwise
        self.m_currentPage = 1
        self.m_path = ""

    def _clearModel(self):
        pass

    def _populateModel(self, json_list):
        pass

    @Property(str)
    def path(self):
        return self.m_path

    @path.setter
    def path(self, p):
        self.m_path = p

    @Property(int, notify=pagesUpdated)
    def pages(self):
        return self.m_pages

    @Property(int, notify=pageUpdated)
    def page(self):
        return self.m_currentPage

    @page.setter
    def page(self, page):
        if self.m_currentPage == page or page < 1:
            return
        self.m_currentPage = page
        self.pageUpdated.emit()
        self.refreshCurrentPage()

    @Slot()
    def refreshCurrentPage(self):
        query = QUrlQuery()
        query.addQueryItem("page", str(self.m_currentPage))
        request = self.m_api.createRequest(self.m_path, query)
        self.m_manager.get(request, self, self.refreshCurrentPageReply)

    def refreshCurrentPageReply(self, reply):
        error = ""
        if reply.isSuccess():
            (json, jsonError) = reply.readJson()
            if json:
                self.refreshRequestFinished(json)
            else:
                error = jsonError.errorString()
        else:
            reply_error = reply.errorString()
            error = reply_error if reply_error else "Network error"

        if error:
            url = reply.networkReply().url().toString()
            print(f'PaginatedResource: request "{url}" failed: "{error}"', file=sys.stderr)
            self.refreshRequestFailed()

    def refreshRequestFinished(self, json):
        json_object = json.object()
        data = json_object.get("data")
        totalPages = json_object.get(totalPagesField)
        currentPage = json_object.get(currentPageField)
        self._populateModel(data if data else [])
        self.m_pages = int(totalPages) if totalPages else 1
        self.m_currentPage = int(currentPage) if currentPage else 1
        self.pageUpdated.emit()
        self.pagesUpdated.emit()
        self.dataUpdated.emit()

    def refreshRequestFailed(self):
        if self.m_currentPage != 1:
            # A failed refresh. If we weren't on page 1, try that.
            # Last resource on currentPage might have been deleted, causing a failure
            self.setPage(1)
        else:
            # Refresh failed and we we're already on page 1 => clear data
            self.m_pages = 0
            self.pagesUpdated.emit()
            self._clearModel()
            self.dataUpdated.emit()

    @Slot("QVariantMap", int)
    def update(self, data, id):
        request = self.m_api.createRequest(f"{self.m_path}/{id}")
        self.m_manager.put(request, data, self, self.updateReply)

    def updateReply(self, reply):
        if reply.isSuccess():
            self.refreshCurrentPage()

    @Slot("QVariantMap")
    def add(self, data):
        request = self.m_api.createRequest(self.m_path)
        self.m_manager.post(request, data, self, self.updateReply)

    @Slot(int)
    def remove(self, id):
        request = self.m_api.createRequest(f"{self.m_path}/{id}")
        self.m_manager.deleteResource(request, self, self.updateReply)


@QmlElement
class PaginatedColorUsersResource(PaginatedResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.m_model = ColorUserModel(self)

    @Property(ColorUserModel, constant=True)
    def model(self):
        return self.m_model

    def _clearModel(self):
        self.m_model.clear()

    def _populateModel(self, json_list):
        self.m_model.set_data(json_list)

    @Slot(str, result=str)
    def avatarForEmail(self, email):
        return self.m_model.avatarForEmail(email)


@QmlElement
class PaginatedColorsResource(PaginatedResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.m_model = ColorModel(self)

    @Property(ColorModel, constant=True)
    def model(self):
        return self.m_model

    def _clearModel(self):
        self.m_model.clear()

    def _populateModel(self, json_list):
        self.m_model.set_data(json_list)



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.timedqguiapplication import TimedQGuiApplication

from PySide6.QtQuick import QQuickView, QQuickItem


class TestBug915(TimedQGuiApplication):
    def testReturnPolicy(self):
        view = QQuickView()

        item1 = QQuickItem()
        item1.setObjectName("Item1")
        item1.setParentItem(view.contentItem())
        self.assertEqual(item1.objectName(), "Item1")  # check if the item still valid

        item2 = QQuickItem()
        item2.setObjectName("Item2")
        item2.setParentItem(view.contentItem())
        item1 = None
        self.assertEqual(item2.objectName(), "Item2")  # check if the item still valid

        view = None


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import enum
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(True)

from testbinding import TestObject, TestQVariantEnum
from PySide6.QtCore import Qt, QKeyCombination
from PySide6.QtGui import QKeySequence, QAction

from helper.usesqapplication import UsesQApplication


class PyTestQVariantEnum(TestQVariantEnum):
    def __init__(self, var_enum):
        super().__init__(var_enum)

    def getRValEnum(self):
        return Qt.Orientation.Vertical

    def channelingEnum(self, rval_enum):
        return (isinstance(rval_enum, enum.Enum)
                and rval_enum == Qt.Orientation.Vertical)


class QVariantTest(UsesQApplication):

    def testQKeySequenceQVariantOperator(self):
        # bug #775
        ks = QKeySequence(Qt.KeyboardModifier.ShiftModifier, Qt.KeyboardModifier.ControlModifier,
                          Qt.Key.Key_P, Qt.Key.Key_R)
        self.assertEqual(TestObject.checkType(ks), 4107)

    def testQKeySequenceMoreVariations(self):
        QAction().setShortcut(Qt.Modifier.CTRL | Qt.Key.Key_B)
        QAction().setShortcut(Qt.Modifier.CTRL | Qt.Modifier.ALT | Qt.Key.Key_B)
        QAction().setShortcut(Qt.Modifier.CTRL | Qt.KeyboardModifier.AltModifier | Qt.Key.Key_B)
        QAction().setShortcut(QKeySequence(QKeyCombination(Qt.Modifier.CTRL | Qt.Key.Key_B)))
        QKeySequence(Qt.Modifier.CTRL | Qt.Key.Key_Q)

    def testEnum(self):
        # Testing C++ class
        testqvariant = TestQVariantEnum(Qt.CheckState.Checked)
        self.assertEqual(testqvariant.getLValEnum(), Qt.CheckState.Checked)
        self.assertIsInstance(testqvariant.getLValEnum(), enum.Enum)
        # in the case where we return a QVariant of C++ enum, it returns a
        # QVariant(int) to Python unless explicitly handled manually by Shiboken
        self.assertEqual(testqvariant.getRValEnum(), 1)
        self.assertEqual(testqvariant.isEnumChanneled(), False)

        # Testing Python child class
        pytestqvariant = PyTestQVariantEnum(Qt.CheckState.Checked)
        self.assertEqual(pytestqvariant.isEnumChanneled(), True)
        # check toInt() conversion works for PyObjectWrapper
        self.assertEqual(PyTestQVariantEnum.getNumberFromQVarEnum(Qt.Orientation.Vertical), 2)
        # check toInt() conversion for IntEnum
        self.assertEqual(PyTestQVariantEnum.getNumberFromQVarEnum(Qt.GestureType.TapGesture), 1)

        # Test if enum still an enum on C++ land
        self.assertTrue(TestQVariantEnum.isQtOrientationEnum(Qt.Orientation.Vertical))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for ...'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import NonDefaultCtor


class DerivedNonDefaultCtor (NonDefaultCtor):
    def returnMyselfVirtual(self):
        return NonDefaultCtor(self.value() + 1)


class AnotherDerivedNonDefaultCtor (NonDefaultCtor):
    def __init__(self, some_string):
        pass


class NonDefaultCtorTest(unittest.TestCase):

    def testNonDefaultCtor(self):
        c = NonDefaultCtor(2)
        # these functions returns NonDefaultCtor by value, so a PyObjecy  is created every time
        self.assertNotEqual(c.returnMyself(), c)
        self.assertEqual(c.returnMyself().value(), 2)
        self.assertNotEqual(c.returnMyself(3), c)
        self.assertEqual(c.returnMyself(3).value(), 2)
        self.assertNotEqual(c.returnMyself(4, c), c)
        self.assertEqual(c.returnMyself(4, c).value(), 2)

    def testVirtuals(self):
        c = DerivedNonDefaultCtor(3)
        # these functions returns NonDefaultCtor by value, so a PyObjecy  is created every time
        self.assertNotEqual(c.returnMyselfVirtual(), c)
        self.assertEqual(c.returnMyselfVirtual().value(), 4)
        self.assertEqual(c.callReturnMyselfVirtual().value(), 4)

    def testCtorOverload(self):
        c = AnotherDerivedNonDefaultCtor("testing")  # noqa: F841


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import contextlib
import difflib
import importlib
import io
import os
import shutil
import sys
import tempfile
import unittest
from pathlib import Path
from unittest import TestCase
from unittest import mock
from unittest.mock import patch

sys.path.append(str(Path(__file__).resolve().parents[2]))
from init_paths import init_test_paths

init_test_paths(False)


def file_diff(expected_file: Path, actual_file: Path) -> str:
    """
    Get a unified diff between two files
    """
    target_text = expected_file.read_text(encoding="utf-8").splitlines()
    generated_text = actual_file.read_text(encoding="utf-8").splitlines()

    return "\n".join(difflib.unified_diff(
        generated_text, target_text,
        fromfile=str(actual_file),
        tofile=str(expected_file),
        lineterm=""
    ))


class PySide6ProjectTestBase(TestCase):
    # If a project name is specified, on each the test, the project folder will be copy to the
    # temp dir and the current dir will be changed to the project folder
    # The project name must match an existing folder in the folder where this file is located
    project_name: str | None = None

    @classmethod
    def setUpClass(cls):
        cls.pyside_root = Path(__file__).parents[5].resolve()
        tools_path = cls.pyside_root / "sources" / "pyside-tools"
        if tools_path not in sys.path:
            sys.path.append(str(tools_path))
        cls.project = importlib.import_module("project")
        cls.project_lib = importlib.import_module("project_lib")
        cls.temp_dir = Path(tempfile.mkdtemp())
        cls.current_dir = Path.cwd()
        # print no outputs to stdout
        sys.stdout = mock.MagicMock()
        if cls.project_name:
            cls.temp_project = Path(cls.temp_dir / cls.project_name).resolve()
        os.chdir(cls.temp_dir)

    def setUp(self):
        super().setUp()
        if self.project_name:
            shutil.copytree(Path(__file__).parent / self.project_name, self.temp_project)
            os.chdir(self.temp_project)

    def tearDown(self):
        super().tearDown()
        if self.project_name:
            os.chdir(self.temp_dir)
            shutil.rmtree(self.temp_project)

    @classmethod
    def tearDownClass(cls):
        os.chdir(cls.current_dir)
        shutil.rmtree(cls.temp_dir)


class TestPySide6ProjectDesignStudio(PySide6ProjectTestBase):
    project_name = "example_drumpad"

    def testDrumpadExample(self):
        # This test compiles the .qrc file into a .py file and checks whether the compilation is
        # carried out only when required
        compiled_resources_path = Path("Python") / "autogen" / "resources.py"
        resources_path = Path("Drumpad.qrc")

        requires_rebuild = self.project_lib.utils.requires_rebuild
        pyproject_path = Path("Python") / "Drumpad.pyproject"
        self.assertFalse(compiled_resources_path.exists())
        self.assertTrue(pyproject_path.exists())
        self.project.main(mode="build", project_path=pyproject_path)

        self.assertTrue(compiled_resources_path.exists())
        self.assertFalse(requires_rebuild([resources_path], compiled_resources_path))

        # Refresh the modification timestamp of the .qrc resources file so that it is considered
        # as modified
        resources_path.touch()

        self.assertTrue(requires_rebuild([resources_path], compiled_resources_path))

        self.project.main(mode="build", project_path=pyproject_path)

        self.assertFalse(requires_rebuild([resources_path], compiled_resources_path))

        # Refresh the modification timestamp of one of the resources files
        list((Path("Resources").glob("*.txt")))[0].touch()
        self.assertTrue(requires_rebuild([resources_path], compiled_resources_path))

        self.project.main(mode="clean", project_path=pyproject_path)

        self.assertFalse(compiled_resources_path.exists())

    def testMigrateDrumpadExample(self):
        # The pyproject.toml file already contains the expected output
        expected_pyproject_toml = Path("Python") / "pyproject.toml"
        expected_pyproject_toml.rename(expected_pyproject_toml.parent / "expected_pyproject.toml")
        existing_pyproject = Path("Python") / "Drumpad.pyproject"

        with self.assertRaises(SystemExit) as context:
            with patch("builtins.input", return_value="y"):
                self.project.main(mode="migrate-pyproject",
                                  project_path=existing_pyproject.as_posix())

        self.assertEqual(0, context.exception.code)
        generated_pyproject_toml = Path("Python") / "pyproject.toml"
        self.assertTrue(generated_pyproject_toml.exists())
        diff = file_diff(expected_pyproject_toml, generated_pyproject_toml)
        self.assertFalse(diff, f"Generated pyproject.toml does not match:\n{diff}")


class TestPySide6ProjectNew(PySide6ProjectTestBase):
    def testNewUi(self):
        test_project_path = self.temp_dir / "NewUiProject"
        with self.assertRaises(SystemExit) as context:
            self.project.main(mode="new-ui", project_dir=test_project_path.as_posix())

        self.assertTrue((test_project_path / "pyproject.toml").exists())
        self.assertTrue((test_project_path / "mainwindow.ui").exists())
        self.assertTrue((test_project_path / "main.py").exists())
        self.assertEqual(0, context.exception.code)
        shutil.rmtree(test_project_path)

    def testRaiseErrorOnExistingNonEmptyProject(self):
        # Create a project twice to ensure that an error is raised
        project_name = "TestProject"
        with self.assertRaises(SystemExit) as context:
            self.project.main(mode="new-ui", project_dir=project_name)

        self.assertEqual(0, context.exception.code)

        error_message = io.StringIO()
        with self.assertRaises(SystemExit) as context:
            with contextlib.redirect_stderr(error_message):
                self.project.main(mode="new-ui", project_dir=project_name)

        self.assertEqual(1, context.exception.code)
        self.assertTrue(f"Can not create project at {project_name}: directory is not empty." in
                        error_message.getvalue())
        shutil.rmtree(self.temp_dir / "TestProject")

    def testRaiseErrorOnInvalidProjectName(self):
        # Create a project with an empty project name
        error_message = io.StringIO()
        with self.assertRaises(SystemExit) as context:
            with contextlib.redirect_stderr(error_message):
                self.project.main(mode="new-ui", project_dir="asdf/?^%$#@!")

        self.assertEqual(1, context.exception.code)
        self.assertTrue("Invalid project name" in error_message.getvalue())

    def testNewQuick(self):
        test_project_path = Path("QuickProject")

        with self.assertRaises(SystemExit) as context:
            self.project.main(mode="new-quick", project_dir=str(test_project_path))

        self.assertTrue((test_project_path / "pyproject.toml").exists())
        self.assertTrue((test_project_path / "main.qml").exists())
        self.assertTrue((test_project_path / "main.py").exists())
        self.assertEqual(0, context.exception.code)
        shutil.rmtree(test_project_path)

    def testNewWidget(self):
        project_dir = self.temp_dir / "inner_folder" / "another_folder" / "WidgetProject"
        with self.assertRaises(SystemExit) as context:
            self.project.main(mode="new-widget", project_dir=project_dir.as_posix())
        self.assertTrue((project_dir / "pyproject.toml").exists())
        self.assertTrue((project_dir / "main.py").exists())
        self.assertEqual(0, context.exception.code)
        shutil.rmtree(project_dir)

    def testRaiseErrorWhenNoProjectNameIsSpecified(self):
        mode = "new-widget"
        error_message = io.StringIO()
        with self.assertRaises(SystemExit) as context, contextlib.redirect_stderr(error_message):
            self.project.main(mode=mode)
        self.assertEqual(context.exception.code, 1)
        expected_msg = f"Error creating new project: {mode} requires a directory name or path"
        self.assertTrue(expected_msg in error_message.getvalue())

    def testCreateProjectLegacyPyProjectFile(self):
        project_path = Path("TestPyProjectJSON")
        mode = "new-widget"
        with self.assertRaises(SystemExit) as context:
            self.project.main(mode=mode, project_dir=project_path.as_posix(), legacy_pyproject=True)
        self.assertEqual(0, context.exception.code)
        self.assertTrue((project_path / "main.py").exists())
        self.assertTrue((project_path / f"{project_path.name}.pyproject").exists())


class TestPySide6ProjectRun(PySide6ProjectTestBase):
    project_name = "example_project"

    def testRaiseErrorWhenRunningEmptyProject(self):
        # Create a new empty project in the temp dir
        project_folder = self.temp_dir / "empty_project"
        project_folder.mkdir()
        os.chdir(project_folder)

        error_message = io.StringIO()
        with self.assertRaises(SystemExit) as context:
            with contextlib.redirect_stderr(error_message):
                self.project.main(mode="run")

        os.chdir(self.temp_dir)
        shutil.rmtree(project_folder)

        self.assertEqual(1, context.exception.code)
        self.assertTrue("No project file found" in error_message.getvalue())

    def testRunExampleProject(self):
        # The project is executed in a subprocess. The proejct code reads the PYSIDE_TESTING
        # environment variable to avoid starting the Qt event loop
        os.environ["PYSIDE_TESTING"] = "1"
        with self.assertRaises(SystemExit) as context:
            self.project.main(mode="run")
        os.environ.pop("PYSIDE_TESTING")
        self.assertEqual(0, context.exception.code)

        self.assertEqual(Path("pyproject.toml").resolve(),
                         self.project_lib.resolve_valid_project_file())


class TestPySide6ProjectExampleProject(PySide6ProjectTestBase):
    """
    Test of an example project with both pyproject.toml and .pyproject valid files.
    Contains a subproject with its own pyproject.toml file and .pyproject file too
    """
    project_name = "example_project"

    def testMigratePyProjectToToml(self):
        # The existing pyproject.toml file contains the expected output
        expected_pyproject_toml = Path("pyproject.toml").rename("expected_pyproject.toml")
        expected_subproject_pyproject_toml = Path("subproject") / "pyproject.toml"
        expected_subproject_pyproject_toml.rename(
            expected_subproject_pyproject_toml.parent / "expected_subproject_pyproject.toml")

        with self.assertRaises(SystemExit) as context:
            self.project.main(mode="migrate-pyproject")

        self.assertEqual(0, context.exception.code)

        generated_pyproject_toml = Path("pyproject.toml")
        self.assertTrue(generated_pyproject_toml.exists())
        diff = file_diff(expected_pyproject_toml, generated_pyproject_toml)
        self.assertFalse(diff, f"Generated pyproject.toml does not match:\n{diff}")

        generated_subproject_pyproject_toml = Path("subproject") / "pyproject.toml"
        self.assertTrue(generated_subproject_pyproject_toml.exists())
        diff = file_diff(expected_subproject_pyproject_toml, generated_subproject_pyproject_toml)
        self.assertFalse(diff, f"Generated subproject/pyproject.toml does not match:\n{diff}")

    def testMigratePyProjectToTomlSpecifyingPyProjectFile(self):
        # The existing pyproject.toml file contains the expected output
        existing_pyproject = Path("example_project.pyproject")
        expected_pyproject_toml = Path("pyproject.toml")
        expected_pyproject_toml.rename("example_project.toml")

        expected_subproject_pyproject_toml = Path("subproject") / "pyproject.toml"
        expected_subproject_pyproject_toml.rename(
            expected_subproject_pyproject_toml.parent / "expected_pyproject.toml")

        with self.assertRaises(SystemExit) as context:
            with patch("builtins.input", return_value="y"):
                self.project.main(mode="migrate-pyproject",
                                  project_path=existing_pyproject.as_posix())

        self.assertEqual(0, context.exception.code)

        generated_pyproject_toml = Path("pyproject.toml")
        self.assertTrue(generated_pyproject_toml.exists())
        diff = file_diff(expected_pyproject_toml, generated_pyproject_toml)
        self.assertFalse(diff, f"Generated pyproject.toml does not match:\n{diff}")

        generated_subproject_pyproject_toml = Path("subproject") / "pyproject.toml"
        self.assertTrue(generated_subproject_pyproject_toml.exists())
        diff = file_diff(expected_subproject_pyproject_toml, generated_subproject_pyproject_toml)
        self.assertFalse(diff, f"Generated subproject/pyproject.toml does not match:\n{diff}")


class TestPySide6ProjectExistingPyProjectToml(PySide6ProjectTestBase):
    """
    Test for migrating a project with an existing pyproject.toml file which does not contain the
    [tool.pyside6-project] section
    """
    project_name = "existing_pyproject_toml"

    def testMigratePyProjectToTomlAlreadyExistingTomlFile(self):
        with self.assertRaises(SystemExit) as context:
            with patch("builtins.input", return_value="y"):
                self.project.main(mode="migrate-pyproject")

        self.assertEqual(0, context.exception.code)
        diff = file_diff(Path("expected_pyproject.toml"),
                         Path("pyproject.toml"))
        self.assertFalse(diff, f"Updated pyproject.toml does not match:\n{diff}")


class TestPySide6ProjectInvalidPyProjectToml(PySide6ProjectTestBase):
    """
    Check the current behavior in a project with an existing invalid pyproject.toml file and
    invalid_pyproject.pyproject file
    """

    project_name = "invalid_pyproject"

    def testRunInvalidPyProjectTomlFile(self):
        pyproject_toml = Path("pyproject.toml")
        self.assertTrue(pyproject_toml.exists())
        self.assertTrue(self.project_lib.parse_pyproject_toml(pyproject_toml).errors)

        error_message = io.StringIO()
        with contextlib.redirect_stderr(error_message):
            with self.assertRaises(SystemExit) as context:
                self.project.main(mode="run", project_path=pyproject_toml.as_posix())

        self.assertEqual(1, context.exception.code)
        self.assertTrue("Invalid project file" in error_message.getvalue())

    def testRunSpecifyingPyProjectJsonFile(self):
        # Check that the *.pyproject file is used if the pyproject.toml is invalid when using
        # pyside6-project run

        pyproject_toml_file = Path("pyproject.toml")
        self.assertTrue(pyproject_toml_file.exists())
        # Ensure that pyproject.toml is considered invalid
        self.assertTrue(self.project_lib.parse_pyproject_toml(pyproject_toml_file).errors)

        valid_pyproject = Path("valid_pyproject.pyproject")
        self.assertTrue(valid_pyproject.exists())

        # Ensure that the project can still be run specifying a valid *.pyproject JSON file
        with self.assertRaises(SystemExit) as context:
            self.project.main(mode="run", project_path=valid_pyproject.as_posix())

        self.assertEqual(0, context.exception.code)
        self.assertTrue(Path("main.py").exists())

    def testErrorRaisesWhenRunningWithoutSpecifyingProjectFile(self):
        # The project folder contains two *.pyproject JSON files.
        # The tool should raise an error because the project file is not specified
        error_message = io.StringIO()
        with contextlib.redirect_stderr(error_message):
            with self.assertRaises(SystemExit) as context:
                self.project.main(mode="run")
        self.assertEqual(1, context.exception.code)
        self.assertTrue("Multiple project files found" in error_message.getvalue())

    def testRaiseErrorResolvingInvalidProjectFile(self):
        # Simulate that the user is specifying an invalid project file
        invalid_pyproject_file = Path("invalid_pyproject.pyproject")
        self.assertTrue(invalid_pyproject_file.exists())

        with self.assertRaises(ValueError) as context:
            self.project_lib.resolve_valid_project_file(invalid_pyproject_file.as_posix())

        exception_message = str(context.exception)
        self.assertTrue("Invalid project file" in exception_message)
        self.assertTrue(str(invalid_pyproject_file) in exception_message)

    def testResolveValidProjectFile(self):
        # Simulate that the user is specifying a valid project file
        valid_pyproject_file = Path("valid_pyproject.pyproject")
        actual_project_file = self.project_lib.resolve_valid_project_file(
            valid_pyproject_file.as_posix())
        self.assertEqual(valid_pyproject_file.resolve(), actual_project_file)

    def testRaiseErrorResolvingInvalidPyProjectToml(self):
        # Simulate that the user is specifying an invalid pyproject.toml file
        pyproject_toml_file = Path("pyproject.toml")
        self.assertTrue(pyproject_toml_file.exists())

        with self.assertRaises(ValueError) as context:
            self.project_lib.resolve_valid_project_file(pyproject_toml_file.as_posix())

        exception_message = str(context.exception)
        self.assertTrue("Invalid project file" in exception_message)
        self.assertTrue(str(pyproject_toml_file) in exception_message)

    def testMigrateInvalidPyProjectToml(self):
        # Can not migrate a project with an invalid pyproject.toml file
        error_message = io.StringIO()
        with contextlib.redirect_stderr(error_message):
            with self.assertRaises(SystemExit) as context:
                with patch("builtins.input", return_value="y"):
                    self.project.main(mode="migrate-pyproject")

        self.assertEqual(1, context.exception.code)
        self.assertTrue("Invalid project file" in error_message.getvalue())

    def testMigrateInvalidPyProjectTomlSpecifyingWrongFile(self):
        # Test specifying the pyproject.toml file as the project file to be migrated
        existing_invalid_pyproject_toml = Path("pyproject.toml")
        self.assertTrue(
            bool(self.project_lib.parse_pyproject_toml(existing_invalid_pyproject_toml).errors))

        error_message = io.StringIO()
        with contextlib.redirect_stderr(error_message):
            with self.assertRaises(SystemExit) as context:
                self.project.main(mode="migrate-pyproject",
                                  project_path=existing_invalid_pyproject_toml)

        self.assertEqual(1, context.exception.code)
        self.assertTrue("Cannot migrate non \"*.pyproject\" file" in error_message.getvalue())
        self.assertTrue("pyproject.toml" in error_message.getvalue())


def testRunInvalidPyProjectToml(self):
    # Ensure that the .pyproject file is preferred over the invalid pyproject.toml file.
    # This preserves the backward compatibility of the .pyproject file

    # Remove the invalid invalid_pyproject.pyproject file first
    Path("invalid_pyproject.pyproject").unlink()
    self.assertFalse(Path("invalid_pyproject.pyproject").exists())

    with self.assertRaises(SystemExit) as context:
        self.project.main(mode="run")

    self.assertEqual(0, context.exception.code)
    self.assertEqual(Path("valid_pyproject.pyproject").resolve(),
                     self.project_lib.resolve_valid_project_file())


class TestPySide6ProjectMultiplePyProject(PySide6ProjectTestBase):
    project_name = "multiple_pyproject"

    def testCancelMigration(self):
        # Ensure that the pyproject.toml is not created if the user cancels the operation
        with self.assertRaises(SystemExit) as context:
            with patch("builtins.input", return_value="n"):
                self.project.main(mode="migrate-pyproject")

        self.assertEqual(0, context.exception.code)
        self.assertFalse(Path("pyproject.toml").exists())

    def testMigrateMultiplePyProjectFilesToToml(self):
        expected_pyproject_toml = Path("expected_pyproject.toml")
        generated_pyproject_toml = Path("pyproject.toml")

        with self.assertRaises(SystemExit) as context:
            with patch("builtins.input", return_value="y"):
                self.project.main(mode="migrate-pyproject")

        self.assertEqual(0, context.exception.code)
        self.assertTrue(generated_pyproject_toml.exists())
        diff = file_diff(expected_pyproject_toml, generated_pyproject_toml)
        self.assertFalse(diff, f"Generated pyproject.toml does not match:\n{diff}")


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtPrintSupport import QPrintDialog, QPrinter
from PySide6.QtWidgets import (QApplication, QDialog, QFileDialog, QLabel,
                               QMainWindow, QMessageBox, QScrollArea,
                               QSizePolicy)
from PySide6.QtGui import (QColorSpace, QGuiApplication,
                           QImageReader, QImageWriter, QKeySequence,
                           QPalette, QPainter, QPixmap)
from PySide6.QtCore import QDir, QStandardPaths, Qt, Slot


ABOUT = """<p>The <b>Image Viewer</b> example shows how to combine QLabel
and QScrollArea to display an image. QLabel is typically used
for displaying a text, but it can also display an image.
QScrollArea provides a scrolling view around another widget.
If the child widget exceeds the size of the frame, QScrollArea
automatically provides scroll bars. </p><p>The example
demonstrates how QLabel's ability to scale its contents
(QLabel.scaledContents), and QScrollArea's ability to
automatically resize its contents
(QScrollArea.widgetResizable), can be used to implement
zooming and scaling features. </p><p>In addition the example
shows how to use QPainter to print an image.</p>
"""


class ImageViewer(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._scale_factor = 1.0
        self._first_file_dialog = True
        self._image_label = QLabel()
        self._image_label.setBackgroundRole(QPalette.ColorRole.Base)
        self._image_label.setSizePolicy(QSizePolicy.Policy.Ignored,
                                        QSizePolicy.Policy.Ignored)
        self._image_label.setScaledContents(True)

        self._scroll_area = QScrollArea()
        self._scroll_area.setBackgroundRole(QPalette.ColorRole.Dark)
        self._scroll_area.setWidget(self._image_label)
        self._scroll_area.setVisible(False)
        self.setCentralWidget(self._scroll_area)

        self._create_actions()

        self.resize(QGuiApplication.primaryScreen().availableSize() * 3 / 5)

    def load_file(self, fileName):
        reader = QImageReader(fileName)
        reader.setAutoTransform(True)
        new_image = reader.read()
        native_filename = QDir.toNativeSeparators(fileName)
        if new_image.isNull():
            error = reader.errorString()
            QMessageBox.information(self, QGuiApplication.applicationDisplayName(),
                                    f"Cannot load {native_filename}: {error}")
            return False
        self._set_image(new_image)
        self.setWindowFilePath(fileName)

        w = self._image.width()
        h = self._image.height()
        d = self._image.depth()
        color_space = self._image.colorSpace()
        description = color_space.description() if color_space.isValid() else 'unknown'
        message = f'Opened "{native_filename}", {w}x{h}, Depth: {d} ({description})'
        self.statusBar().showMessage(message)
        return True

    def _set_image(self, new_image):
        self._image = new_image
        if self._image.colorSpace().isValid():
            color_space = QColorSpace(QColorSpace.NamedColorSpace.SRgb)
            self._image.convertToColorSpace(color_space)
        self._image_label.setPixmap(QPixmap.fromImage(self._image))
        self._scale_factor = 1.0

        self._scroll_area.setVisible(True)
        self._print_act.setEnabled(True)
        self._fit_to_window_act.setEnabled(True)
        self._update_actions()

        if not self._fit_to_window_act.isChecked():
            self._image_label.adjustSize()

    def _save_file(self, fileName):
        writer = QImageWriter(fileName)

        native_filename = QDir.toNativeSeparators(fileName)
        if not writer.write(self._image):
            error = writer.errorString()
            message = f"Cannot write {native_filename}: {error}"
            QMessageBox.information(self, QGuiApplication.applicationDisplayName(),
                                    message)
            return False
        self.statusBar().showMessage(f'Wrote "{native_filename}"')
        return True

    @Slot()
    def _open(self):
        dialog = QFileDialog(self, "Open File")
        self._initialize_image_filedialog(dialog, QFileDialog.AcceptMode.AcceptOpen)
        while (dialog.exec() == QDialog.DialogCode.Accepted
               and not self.load_file(dialog.selectedFiles()[0])):
            pass

    @Slot()
    def _save_as(self):
        dialog = QFileDialog(self, "Save File As")
        self._initialize_image_filedialog(dialog, QFileDialog.AcceptMode.AcceptSave)
        while (dialog.exec() == QDialog.DialogCode.Accepted
               and not self._save_file(dialog.selectedFiles()[0])):
            pass

    @Slot()
    def _print_(self):
        printer = QPrinter()
        dialog = QPrintDialog(printer, self)
        if dialog.exec() == QDialog.DialogCode.Accepted:
            with QPainter(printer) as painter:
                pixmap = self._image_label.pixmap()
                rect = painter.viewport()
                size = pixmap.size()
                size.scale(rect.size(), Qt.KeepAspectRatio)
                painter.setViewport(rect.x(), rect.y(), size.width(), size.height())
                painter.setWindow(pixmap.rect())
                painter.drawPixmap(0, 0, pixmap)

    @Slot()
    def _copy(self):
        QGuiApplication.clipboard().setImage(self._image)

    @Slot()
    def _paste(self):
        new_image = QGuiApplication.clipboard().image()
        if new_image.isNull():
            self.statusBar().showMessage("No image in clipboard")
        else:
            self._set_image(new_image)
            self.setWindowFilePath('')
            w = new_image.width()
            h = new_image.height()
            d = new_image.depth()
            message = f"Obtained image from clipboard, {w}x{h}, Depth: {d}"
            self.statusBar().showMessage(message)

    @Slot()
    def _zoom_in(self):
        self._scale_image(1.25)

    @Slot()
    def _zoom_out(self):
        self._scale_image(0.8)

    @Slot()
    def _normal_size(self):
        self._image_label.adjustSize()
        self._scale_factor = 1.0

    @Slot()
    def _fit_to_window(self):
        fit_to_window = self._fit_to_window_act.isChecked()
        self._scroll_area.setWidgetResizable(fit_to_window)
        if not fit_to_window:
            self._normal_size()
        self._update_actions()

    @Slot()
    def _about(self):
        QMessageBox.about(self, "About Image Viewer", ABOUT)

    def _create_actions(self):
        file_menu = self.menuBar().addMenu("&File")

        self._open_act = file_menu.addAction("&Open...")
        self._open_act.triggered.connect(self._open)
        self._open_act.setShortcut(QKeySequence.StandardKey.Open)

        self._save_as_act = file_menu.addAction("&Save As...")
        self._save_as_act.triggered.connect(self._save_as)
        self._save_as_act.setEnabled(False)

        self._print_act = file_menu.addAction("&Print...")
        self._print_act.triggered.connect(self._print_)
        self._print_act.setShortcut(QKeySequence.StandardKey.Print)
        self._print_act.setEnabled(False)

        file_menu.addSeparator()

        self._exit_act = file_menu.addAction("E&xit")
        self._exit_act.triggered.connect(self.close)
        self._exit_act.setShortcut("Ctrl+Q")

        edit_menu = self.menuBar().addMenu("&Edit")

        self._copy_act = edit_menu.addAction("&Copy")
        self._copy_act.triggered.connect(self._copy)
        self._copy_act.setShortcut(QKeySequence.StandardKey.Copy)
        self._copy_act.setEnabled(False)

        self._paste_act = edit_menu.addAction("&Paste")
        self._paste_act.triggered.connect(self._paste)
        self._paste_act.setShortcut(QKeySequence.StandardKey.Paste)

        view_menu = self.menuBar().addMenu("&View")

        self._zoom_in_act = view_menu.addAction("Zoom &In (25%)")
        self._zoom_in_act.setShortcut(QKeySequence.StandardKey.ZoomIn)
        self._zoom_in_act.triggered.connect(self._zoom_in)
        self._zoom_in_act.setEnabled(False)

        self._zoom_out_act = view_menu.addAction("Zoom &Out (25%)")
        self._zoom_out_act.triggered.connect(self._zoom_out)
        self._zoom_out_act.setShortcut(QKeySequence.StandardKey.ZoomOut)
        self._zoom_out_act.setEnabled(False)

        self._normal_size_act = view_menu.addAction("&Normal Size")
        self._normal_size_act.triggered.connect(self._normal_size)
        self._normal_size_act.setShortcut("Ctrl+S")
        self._normal_size_act.setEnabled(False)

        view_menu.addSeparator()

        self._fit_to_window_act = view_menu.addAction("&Fit to Window")
        self._fit_to_window_act.triggered.connect(self._fit_to_window)
        self._fit_to_window_act.setEnabled(False)
        self._fit_to_window_act.setCheckable(True)
        self._fit_to_window_act.setShortcut("Ctrl+F")

        help_menu = self.menuBar().addMenu("&Help")

        about_act = help_menu.addAction("&About")
        about_act.triggered.connect(self._about)
        about_qt_act = help_menu.addAction("About &Qt")
        about_qt_act.triggered.connect(QApplication.aboutQt)

    def _update_actions(self):
        has_image = not self._image.isNull()
        self._save_as_act.setEnabled(has_image)
        self._copy_act.setEnabled(has_image)
        enable_zoom = not self._fit_to_window_act.isChecked()
        self._zoom_in_act.setEnabled(enable_zoom)
        self._zoom_out_act.setEnabled(enable_zoom)
        self._normal_size_act.setEnabled(enable_zoom)

    def _scale_image(self, factor):
        self._scale_factor *= factor
        new_size = self._scale_factor * self._image_label.pixmap().size()
        self._image_label.resize(new_size)

        self._adjust_scrollbar(self._scroll_area.horizontalScrollBar(), factor)
        self._adjust_scrollbar(self._scroll_area.verticalScrollBar(), factor)

        self._zoom_in_act.setEnabled(self._scale_factor < 3.0)
        self._zoom_out_act.setEnabled(self._scale_factor > 0.333)

    def _adjust_scrollbar(self, scrollBar, factor):
        pos = int(factor * scrollBar.value()
                  + ((factor - 1) * scrollBar.pageStep() / 2))
        scrollBar.setValue(pos)

    def _initialize_image_filedialog(self, dialog, acceptMode):
        if self._first_file_dialog:
            self._first_file_dialog = False
            locations = QStandardPaths.standardLocations(QStandardPaths.StandardLocation.PicturesLocation)  # noqa: E501
            directory = locations[-1] if locations else QDir.currentPath()
            dialog.setDirectory(directory)

        mime_types = [m.data().decode('utf-8') for m in QImageWriter.supportedMimeTypes()]
        mime_types.sort()

        dialog.setMimeTypeFilters(mime_types)
        dialog.selectMimeTypeFilter("image/jpeg")
        dialog.setAcceptMode(acceptMode)
        if acceptMode == QFileDialog.AcceptMode.AcceptSave:
            dialog.setDefaultSuffix("jpg")



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

import sys
from pathlib import Path

from PySide6.QtCore import QCoreApplication, QUrl, QRect, QPoint
from PySide6.QtGui import QGuiApplication
from PySide6.QtQml import QQmlApplicationEngine
from PySide6.QtWebView import QtWebView
import argparse

import rc_qml  # noqa: F401


class Utils:
    @staticmethod
    def fromUserInput(userInput):
        if not userInput:
            return QUrl.fromUserInput("about:blank")
        result = QUrl.fromUserInput(userInput)
        return result if result.isValid() else QUrl.fromUserInput("about:blank")


if __name__ == "__main__":
    QtWebView.initialize()
    app = QGuiApplication(sys.argv)
    QGuiApplication.setApplicationDisplayName(QCoreApplication
                                              .translate("main", "QtWebView Example"))

    parser = argparse.ArgumentParser(description=QGuiApplication.applicationDisplayName())
    parser.add_argument("--url", nargs="?",
                        default="https://www.qt.io",
                        help="The initial URL to open.")
    args = parser.parse_args()
    initialUrl = args.url

    engine = QQmlApplicationEngine()
    context = engine.rootContext()
    context.setContextProperty("utils", Utils())
    context.setContextProperty("initialUrl", Utils.fromUserInput(initialUrl))

    geometry = QGuiApplication.primaryScreen().availableGeometry()
    if not QGuiApplication.styleHints().showIsFullScreen():
        size = geometry.size() * 4 / 5
        offset = (geometry.size() - size) / 2
        pos = geometry.topLeft() + QPoint(offset.width(), offset.height())
        geometry = QRect(pos, size)

    engine.setInitialProperties({"x": geometry.x(), "y": geometry.y(),
                                 "width": geometry.width(), "height": geometry.height()})
    qml_file = Path(__file__).parent / "main.qml"
    engine.load(QUrl.fromLocalFile(qml_file))

    if not engine.rootObjects():
        sys.exit(-1)

    exit_code = app.exec()
    del engine
    sys.exit(exit_code)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
from itertools import chain

from PySide6.QtCore import QAbstractTableModel, Qt, Signal, Slot
from PySide6.QtWidgets import QApplication, QMainWindow, QTableView

"""PySide6 port of the widgets/tutorials/modelview/5_edit example from Qt v6.x"""


COLS = 3
ROWS = 2


class MyModel(QAbstractTableModel):

    editCompleted = Signal(str)

    def __init__(self, parent=None):
        super().__init__(parent)
        self._grid_data = [["" for y in range(COLS)] for x in range(ROWS)]

    def rowCount(self, parent=None):
        return ROWS

    def columnCount(self, parent=None):
        return COLS

    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
        if role == Qt.ItemDataRole.DisplayRole and self.checkIndex(index):
            return self._grid_data[index.row()][index.column()]
        return None

#! [1]
    def setData(self, index, value, role):
        if role != Qt.ItemDataRole.EditRole or not self.checkIndex(index):
            return False
        # save value from editor to member m_gridData
        self._grid_data[index.row()][index.column()] = value
        # for presentation purposes only: build and emit a joined string
        result = " ".join(chain(*self._grid_data))
        self.editCompleted.emit(result)
        return True
#! [1]

#! [2]
    def flags(self, index):
        return Qt.ItemIsEditable | super().flags(index)
#! [2]


class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._table_view = QTableView(self)
        self.setCentralWidget(self._table_view)
        my_model = MyModel(self)
        self._table_view.setModel(my_model)
        # transfer changes to the model to the window title
        my_model.editCompleted.connect(self.show_window_title)

    @Slot(str)
    def show_window_title(self, title):
        self.setWindowTitle(title)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    w = MainWindow()
    w.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication

from PySide6.QtWidgets import QTreeWidget, QTreeWidgetItem, QTreeWidgetItemIterator


class QTreeWidgetItemIteratorTest(UsesQApplication):
    def testWidgetIterator(self):
        treeWidget = QTreeWidget()
        treeWidget.setColumnCount(1)
        items = []
        for i in range(10):
            items.append(QTreeWidgetItem(None, [f'item: {i}']))
        treeWidget.insertTopLevelItems(0, items)

        index = 0
        for it in QTreeWidgetItemIterator(treeWidget):
            self.assertEqual(it.value().text(0), f'item: {index}')
            index += 1


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import inspect
import os
import re
import subprocess
import sys
from subprocess import TimeoutExpired
from textwrap import dedent

# Get the dir path to the utils module
try:
    this_file = __file__
except NameError:
    this_file = sys.argv[0]
this_file = os.path.abspath(this_file)
this_dir = os.path.dirname(this_file)
build_scripts_dir = os.path.abspath(os.path.join(this_dir, ".."))

sys.path.append(build_scripts_dir)
from build_scripts.utils import detect_clang    # noqa: E402


class TestRunner:
    def __init__(self, log_entry, project, index):
        self.log_entry = log_entry
        built_path = log_entry.build_dir
        self.test_dir = os.path.join(built_path, project)
        log_dir = log_entry.log_dir
        if index is not None:
            self.logfile = os.path.join(log_dir, f"{project}.{index}.log")
        else:
            self.logfile = os.path.join(log_dir, f"{project}.log")
        os.environ["CTEST_OUTPUT_ON_FAILURE"] = "1"
        self._setup_clang()
        self._setup()

    def get_python_version(self):
        """
        Finding the exact Python version.
        ---------------------------------

        This is done by asking the interpreter, because it cannot reliably
        be found from any file name parsing as a triple.

        Note: We need to look into the CMakeCache.txt file to find out
        what CMake has found as the Python interpreter to use.
        This is *not* necessarily the same Python that runs this script,
        otherwise we could use the version info directly.
        """
        look_python = os.path.join(self.test_dir, "CMakeCache.txt")
        look_for = "PYTHON_EXECUTABLE:FILEPATH="
        with open(look_python) as f:
            for line in f:
                if line.startswith(look_for):
                    python_exec = line.split("=")[-1].strip()
                    res = subprocess.run([python_exec, "-c",
                                         "import sys;print(sys.version_info[:3])"],
                                         capture_output=True)
                    return eval(res.stdout.decode("utf-8"))
        return None

    def _setup_clang(self):
        if sys.platform != "win32":
            return
        clang_dir = detect_clang()
        if clang_dir[0]:
            clang_bin_dir = os.path.join(clang_dir[0], "bin")
            path = os.environ.get("PATH")
            if clang_bin_dir not in path:
                os.environ["PATH"] = clang_bin_dir + os.pathsep + path
                print(f"Adding {clang_bin_dir} as detected by {clang_dir[1]} to PATH")

    def _find_ctest_in_file(self, file_name):
        """
        Helper for _find_ctest() that finds the ctest binary in a build
        system file (ninja, Makefile).
        """
        # Looking for a command ending this way:
        look_for = "\\ctest.exe" if "win32" in sys.platform else "/ctest"
        line = None
        with open(file_name) as makefile:
            for line in makefile:
                if look_for in line and line.lstrip().startswith("COMMAND"):
                    break
            else:
                # We have probably forgotten to build the tests.
                # Give a nice error message with a shortened but exact path.
                rel_path = os.path.relpath(file_name)
                msg = dedent(
                    f"""\n
                    {'*' * 79}
                    **  ctest is not in '{rel_path}'.
                    *   Did you forget to build the tests with '--build-tests' in setup.py?
                    """
                )
                raise RuntimeError(msg)
        # the ctest program is on the left to look_for
        assert line, f"Did not find {look_for}"
        look = re.escape(look_for)
        ctest = re.search(fr'(\S+{look}|"([^"]+{look})")', line).groups()
        return ctest[1] or ctest[0]

    def _find_ctest(self):
        """
        Find ctest in a build system file (ninja, Makefile)

        We no longer use make, but the ctest command directly.
        It is convenient to look for the ctest program using the Makefile.
        This serves us two purposes:

          - there is no dependency of the PATH variable,
          - each project is checked whether ctest was configured.
        """
        candidate_files = ["Makefile", "build.ninja"]
        for candidate in candidate_files:
            path = os.path.join(self.test_dir, candidate)
            if os.path.exists(path):
                return self._find_ctest_in_file(path)
        raise RuntimeError(
            "Cannot find any of the build system files " f"{', '.join(candidate_files)}."
        )

    def _setup(self):
        self.ctestCommand = self._find_ctest()

    def _run(self, cmd_tuple, label, timeout):
        """
        Perform a test run in a given build

        The build can be stopped by a keyboard interrupt for testing
        this script. Also, a timeout can be used.

        After the change to directly using ctest, we no longer use
        "--force-new-ctest-process". Until now this has no drawbacks
        but was a little faster.
        """

        self.cmd = cmd_tuple
        # We no longer use the shell option. It introduces wrong handling
        # of certain characters which are not yet correctly escaped:
        # Especially the "^" caret char is treated as an escape, and pipe symbols
        # without a caret are interpreted as such which leads to weirdness.
        # Since we have all commands with explicit paths and don't use shell
        # commands, this should work fine.
        print(
            dedent(
                f"""\
            running {self.cmd}
                 in {self.test_dir}
            """
            )
        )
        ctest_process = subprocess.Popen(
            self.cmd, cwd=self.test_dir, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

        def py_tee(input, output, label):
            """
            A simple (incomplete) tee command in Python

            This script simply logs everything from input to output
            while the output gets some decoration. The specific reason
            to have this script at all is:

            - it is necessary to have some decoration as prefix, since
              we run commands several times

            - collecting all output and then decorating is not nice if
              you have to wait for a long time

            The special escape is for the case of an embedded file in
            the output.
            """

            def xprint(*args, **kw):
                print(*args, file=output, **kw)

            # 'for line in input:' would read into too large chunks
            labelled = True
            # make sure that this text is not found in a traceback of the runner!
            text_a = "BEGIN" "_FILE"
            text_z = "END" "_FILE"
            while True:
                line = input.readline()
                if not line:
                    break
                if line.startswith(text_a):
                    labelled = False
                txt = line.rstrip()
                xprint(label, txt) if label and labelled else xprint(txt)
                if line.startswith(text_z):
                    labelled = True

        tee_src = dedent(inspect.getsource(py_tee))
        tee_src = f"import sys\n{tee_src}\npy_tee(sys.stdin, sys.stdout, '{label}')"
        tee_cmd = (sys.executable, "-E", "-u", "-c", tee_src)
        tee_process = subprocess.Popen(tee_cmd, cwd=self.test_dir, stdin=ctest_process.stdout)
        try:
            comm = tee_process.communicate
            _ = comm(timeout=timeout)[0]
        except (TimeoutExpired, KeyboardInterrupt):
            print()
            print("aborted, partial result")
            ctest_process.kill()
            _ = ctest_process.communicate()
            # ctest lists to a temp file. Move it to the log
            tmp_name = f"{self.logfile}.tmp"
            if os.path.exists(tmp_name):
                if os.path.exists(self.logfile):
                    os.unlink(self.logfile)
                os.rename(tmp_name, self.logfile)
            self.partial = True
        else:
            _ = ctest_process.communicate()
            self.partial = False
        finally:
            print("End of the test run")
            print()
        tee_process.wait()
        ctest_process.wait()

    def run(self, label, rerun, timeout):
        cmd = self.ctestCommand, "--output-log", self.logfile
        if rerun is not None:
            # cmd += ("--rerun-failed",)
            # For some reason, this worked never in the script file.
            # We pass instead the test names as a regex:
            words = "^(" + "|".join(rerun) + ")$"
            cmd += ("--tests-regex", words)
        self._run(cmd, label, timeout)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the qml/examples/qml/referenceexamples/extended example from Qt v6.x"""

from pathlib import Path
import sys

from PySide6.QtCore import QObject, QUrl, Property
from PySide6.QtWidgets import QApplication, QLineEdit
from PySide6.QtQml import (QQmlComponent, QQmlEngine, QmlForeign, QmlExtended,
                           QmlNamedElement)


# To be used on the @QmlElement decorator
# (QML_IMPORT_MINOR_VERSION is optional)
QML_IMPORT_NAME = "examples.extend"
QML_IMPORT_MAJOR_VERSION = 1


class LineEditExtension(QObject):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._line_edit = parent

    @Property(int)
    def left_margin(self):
        return self._line_edit.textMargins().left()

    @left_margin.setter
    def left_margin(self, m):
        margins = self._line_edit.textMargins()
        margins.setLeft(m)
        self._line_edit.setTextMargins(margins)

    @Property(int)
    def right_margin(self):
        return self._line_edit.textMargins().right()

    @right_margin.setter
    def right_margin(self, m):
        margins = self._line_edit.textMargins()
        margins.setRight(m)
        self._line_edit.setTextMargins(margins)

    @Property(int)
    def top_margin(self):
        return self._line_edit.textMargins().top()

    @top_margin.setter
    def top_margin(self, m):
        margins = self._line_edit.textMargins()
        margins.setTop(m)
        self._line_edit.setTextMargins(margins)

    @Property(int)
    def bottom_margin(self):
        return self._line_edit.textMargins().bottom()

    @bottom_margin.setter
    def bottom_margin(self, m):
        margins = self._line_edit.textMargins()
        margins.setBottom(m)
        self._line_edit.setTextMargins(margins)


@QmlNamedElement("QLineEdit")
@QmlExtended(LineEditExtension)
@QmlForeign(QLineEdit)
class LineEditForeign(QObject):

    def __init__(self, parent=None):
        super().__init__(parent)


if __name__ == '__main__':
    app = QApplication(sys.argv)

    qml_file = Path(__file__).parent / "example.qml"
    url = QUrl.fromLocalFile(qml_file)
    engine = QQmlEngine()
    component = QQmlComponent(engine, url)
    widget = component.create()
    if not widget:
        print(component.errors())
        del engine
        sys.exit(-1)

    widget.show()
    r = app.exec()
    # Deleting the engine before it goes out of scope is required to make sure
    # all child QML instances are destroyed in the correct order.
    del engine
    sys.exit(r)



# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the network/fortuneserver example from Qt v5.x"""

import random
import sys

from PySide6.QtCore import QByteArray, QDataStream, QIODevice, Qt
from PySide6.QtNetwork import QTcpServer
from PySide6.QtWidgets import (QApplication, QDialog, QHBoxLayout,
                               QLabel, QMessageBox, QPushButton,
                               QVBoxLayout)


class Server(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)

        status_label = QLabel()
        status_label.setTextInteractionFlags(Qt.TextInteractionFlag.TextBrowserInteraction)
        quit_button = QPushButton("Quit")
        quit_button.setAutoDefault(False)

        self._tcp_server = QTcpServer(self)
        if not self._tcp_server.listen():
            reason = self._tcp_server.errorString()
            QMessageBox.critical(self, "Fortune Server",
                                 f"Unable to start the server: {reason}.")
            self.close()
            return
        port = self._tcp_server.serverPort()
        status_label.setText(f"The server is running on port {port}.\nRun the "
                             "Fortune Client example now.")

        self.fortunes = (
            "You've been leading a dog's life. Stay off the furniture.",
            "You've got to think about tomorrow.",
            "You will be surprised by a loud noise.",
            "You will feel hungry again in another hour.",
            "You might have mail.",
            "You cannot kill time without injuring eternity.",
            "Computers are not intelligent. They only think they are.")

        quit_button.clicked.connect(self.close)
        self._tcp_server.newConnection.connect(self.send_fortune)

        button_layout = QHBoxLayout()
        button_layout.addStretch(1)
        button_layout.addWidget(quit_button)
        button_layout.addStretch(1)

        main_layout = QVBoxLayout(self)
        main_layout.addWidget(status_label)
        main_layout.addLayout(button_layout)

        self.setWindowTitle("Fortune Server")

    def send_fortune(self):
        block = QByteArray()
        out = QDataStream(block, QIODevice.OpenModeFlag.WriteOnly)
        out.setVersion(QDataStream.Version.Qt_4_0)
        out.writeUInt16(0)
        fortune = self.fortunes[random.randint(0, len(self.fortunes) - 1)]

        out.writeString(fortune)
        out.device().seek(0)
        out.writeUInt16(block.size() - 2)

        client_connection = self._tcp_server.nextPendingConnection()
        client_connection.disconnected.connect(client_connection.deleteLater)

        client_connection.write(block)
        client_connection.disconnectFromHost()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    server = Server()
    random.seed(None)
    sys.exit(server.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest
import weakref

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import Slot, QTimer
from PySide6.QtWidgets import QApplication, QDialog, QMainWindow
from helper.timedqapplication import TimedQApplication


def is_exposed(widget):
    result = False
    if widget.isVisible():
        handle = widget.windowHandle()
        if handle:
            result = handle.isExposed()
    return result


class Window(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("Main")
        self.dialog = None
        self._timer = QTimer()
        self._timer.setInterval(50)
        self._timer.timeout.connect(self._timer_handler)
        self._timer.start()

    @Slot()
    def execDialog(self):
        dialog = QDialog(self)
        self.dialog = weakref.ref(dialog)
        dialog.setWindowTitle("Dialog")
        dialog.setMinimumWidth(200)
        QTimer.singleShot(500, dialog.reject)
        dialog.exec()
        self.close()

    @Slot()
    def _timer_handler(self):
        """Periodically check for the dialog to appear and close it."""
        for widget in QApplication.topLevelWidgets():
            if isinstance(widget, QDialog) and is_exposed(widget):
                widget.reject()


class DialogExecTest(TimedQApplication):
    """Test whether the parent-child relationship (dialog/main window) is removed when
       using QDialog.exec() (instead show()), preventing the dialog from leaking."""

    def setUp(self):
        super().setUp(10000)
        self._window = Window()

    def testExec(self):
        self._window.show()
        while not is_exposed(self._window):
            QApplication.processEvents()
        QTimer.singleShot(0, self._window.execDialog)
        self.app.exec()
        self.assertTrue(self._window.dialog() is None)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtCore import QSizeF, Qt
from PySide6.QtWidgets import (QApplication, QGraphicsAnchorLayout,
                               QGraphicsProxyWidget, QGraphicsScene,
                               QGraphicsView, QGraphicsWidget,
                               QPushButton, QSizePolicy)


def create_item(minimum, preferred, maximum, name):
    w = QGraphicsProxyWidget()

    w.setWidget(QPushButton(name))
    w.setMinimumSize(minimum)
    w.setPreferredSize(preferred)
    w.setMaximumSize(maximum)
    w.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)

    return w


if __name__ == '__main__':
    app = QApplication(sys.argv)

    scene = QGraphicsScene()
    scene.setSceneRect(0, 0, 800, 480)

    min_size = QSizeF(30, 100)
    pref_size = QSizeF(210, 100)
    max_size = QSizeF(300, 100)

    a = create_item(min_size, pref_size, max_size, "A")
    b = create_item(min_size, pref_size, max_size, "B")
    c = create_item(min_size, pref_size, max_size, "C")
    d = create_item(min_size, pref_size, max_size, "D")
    e = create_item(min_size, pref_size, max_size, "E")
    f = create_item(QSizeF(30, 50), QSizeF(150, 50), max_size, "F")
    g = create_item(QSizeF(30, 50), QSizeF(30, 100), max_size, "G")

    l = QGraphicsAnchorLayout()  # noqa: E741
    l.setSpacing(0)

    w = QGraphicsWidget(None, Qt.WindowType.Window)
    w.setPos(20, 20)
    w.setLayout(l)

    # Vertical
    l.addAnchor(a, Qt.AnchorPoint.AnchorTop, l, Qt.AnchorPoint.AnchorTop)
    l.addAnchor(b, Qt.AnchorPoint.AnchorTop, l, Qt.AnchorPoint.AnchorTop)

    l.addAnchor(c, Qt.AnchorPoint.AnchorTop, a, Qt.AnchorPoint.AnchorBottom)
    l.addAnchor(c, Qt.AnchorPoint.AnchorTop, b, Qt.AnchorPoint.AnchorBottom)
    l.addAnchor(c, Qt.AnchorPoint.AnchorBottom, d, Qt.AnchorPoint.AnchorTop)
    l.addAnchor(c, Qt.AnchorPoint.AnchorBottom, e, Qt.AnchorPoint.AnchorTop)

    l.addAnchor(d, Qt.AnchorPoint.AnchorBottom, l, Qt.AnchorPoint.AnchorBottom)
    l.addAnchor(e, Qt.AnchorPoint.AnchorBottom, l, Qt.AnchorPoint.AnchorBottom)

    l.addAnchor(c, Qt.AnchorPoint.AnchorTop, f, Qt.AnchorPoint.AnchorTop)
    l.addAnchor(c, Qt.AnchorPoint.AnchorVerticalCenter, f, Qt.AnchorPoint.AnchorBottom)
    l.addAnchor(f, Qt.AnchorPoint.AnchorBottom, g, Qt.AnchorPoint.AnchorTop)
    l.addAnchor(c, Qt.AnchorPoint.AnchorBottom, g, Qt.AnchorPoint.AnchorBottom)

    # Horizontal.
    l.addAnchor(l, Qt.AnchorPoint.AnchorLeft, a, Qt.AnchorPoint.AnchorLeft)
    l.addAnchor(l, Qt.AnchorPoint.AnchorLeft, d, Qt.AnchorPoint.AnchorLeft)
    l.addAnchor(a, Qt.AnchorPoint.AnchorRight, b, Qt.AnchorPoint.AnchorLeft)

    l.addAnchor(a, Qt.AnchorPoint.AnchorRight, c, Qt.AnchorPoint.AnchorLeft)
    l.addAnchor(c, Qt.AnchorPoint.AnchorRight, e, Qt.AnchorPoint.AnchorLeft)

    l.addAnchor(b, Qt.AnchorPoint.AnchorRight, l, Qt.AnchorPoint.AnchorRight)
    l.addAnchor(e, Qt.AnchorPoint.AnchorRight, l, Qt.AnchorPoint.AnchorRight)
    l.addAnchor(d, Qt.AnchorPoint.AnchorRight, e, Qt.AnchorPoint.AnchorLeft)

    l.addAnchor(l, Qt.AnchorPoint.AnchorLeft, f, Qt.AnchorPoint.AnchorLeft)
    l.addAnchor(l, Qt.AnchorPoint.AnchorLeft, g, Qt.AnchorPoint.AnchorLeft)
    l.addAnchor(f, Qt.AnchorPoint.AnchorRight, g, Qt.AnchorPoint.AnchorRight)

    scene.addItem(w)
    scene.setBackgroundBrush(Qt.GlobalColor.darkGreen)

    view = QGraphicsView(scene)
    view.show()

    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import os
import re
from dataclasses import dataclass
from io import StringIO

"""
testing/parser.py

Parse test output lines from ctest and build TestResult objects.

TestParser.iter_blacklist adds info from the blacklist while iterating
over the test results.
"""

_EXAMPLE = """
Example output:

ip1 n        sharp mod_name                                            code      tim
-----------------------------------------------------------------------------------------
114/391 Test #114: QtCore_qfileinfo_test-42 ........................   Passed    0.10 sec
        Start 115: QtCore_qfile_test
115/391 Test #115: QtCore_qfile_test ...............................***Failed    0.11 sec
        Start 116: QtCore_qflags_test

We will only look for the dotted lines and calculate everything from that.
The summary statistics at the end will be ignored. That allows us to test
this functionality with short timeout values.

Note the field "mod_name". I had split this before, but it is necessary
to use the combination as the key, because the test names are not unique.
"""

_TEST_PAT_PRE = r"""
    ^                          # start
    \s*                        # any whitespace ==: WS
    ([0-9]+)/([0-9]+)          #                         ip1 "/" n
    \s+                        # some WS
    Test                       #                         "Test"
    \s+                        # some WS
    \#                         # sharp symbol            "#"
    ([0-9]+)                   #                         sharp
    :                          # colon symbol            ':'
    """
_TEST_PAT = (
    _TEST_PAT_PRE
    + r"""
    \s+                        # some WS
    ([\w-]+)                   #                         mod_name
    .*?                        # whatever (non greedy)
    (                          #
      (Passed)                 # either "Passed", None
      |                        #
      \*\*\*(\w+.*?)           # or     None, "Something"
    )                          #                         code
    \s+                        # some WS
    ([0-9]+\.[0-9]+)           #                         tim
    \s+                        # some WS
    sec                        #                         "sec"
    \s*                        # any WS
    $                          # end
    """
)

# validation of our pattern:
assert re.match(_TEST_PAT, _EXAMPLE.splitlines()[5], re.VERBOSE)
assert len(re.match(_TEST_PAT, _EXAMPLE.splitlines()[5], re.VERBOSE).groups()) == 8
assert len(re.match(_TEST_PAT, _EXAMPLE.splitlines()[7], re.VERBOSE).groups()) == 8


@dataclass
class TestResult:
    idx: int = 0
    n: int = 0
    sharp: int = 0
    mod_name: str = ""
    passed: bool = False
    code: str = ""
    time: float = 0
    fatal: bool = False
    rich_result: str = ""


def sort_time_key(item):
    return item.time


def _parse_tests(test_log):
    """
    Create a TestResult object for every entry.
    """
    result = []
    if isinstance(test_log, StringIO):
        lines = test_log.readlines()
    elif test_log is not None and os.path.exists(test_log):
        with open(test_log) as f:
            lines = f.readlines()
    else:
        lines = []

    # PYSIDE-1229: Fix disrupted lines like "Exit code 0xc0000409\n***Exception:"
    pat = _TEST_PAT_PRE
    for idx, line in enumerate(lines[:-1]):
        match = re.match(pat, line, re.VERBOSE)
        if match and line.split()[-1] != "sec":
            # don't change the number of lines
            lines[idx:idx + 2] = [line.rstrip() + lines[idx + 1], ""]

    pat = _TEST_PAT
    for line in lines:
        match = re.match(pat, line, re.VERBOSE)
        if match:
            idx, n, sharp, mod_name, much_stuff, code1, code2, tim = match.groups()
            # either code1 or code2 is None
            code = code1 or code2
            idx, n, sharp, code, tim = int(idx), int(n), int(sharp), code.lower(), float(tim)
            item = TestResult(idx, n, sharp, mod_name, code == "passed", code, tim, False, None)
            result.append(item)

    # PYSIDE-1229: Be sure that the numbering of the tests is consecutive
    for idx, item in enumerate(result):
        # testing fatal error:
        # Use "if idx + 1 != item.idx or idx == 42:"
        if idx + 1 != item.idx:
            # The numbering is disrupted. Provoke an error in this line!
            code = f"{code}, but lines are disrupted!"
            result[idx].passed = False
            result[idx].code = f"{item.code}, but lines are disrupted!"
            result[idx].fatal = True
            break
    return result


class TestParser:
    def __init__(self, test_log):
        self._results = _parse_tests(test_log)

    @property
    def results(self):
        return self._results

    def __len__(self):
        return len(self._results)

    def iter_blacklist(self, blacklist):
        bl = blacklist
        for item in self.results:
            passed = item.passed
            match = bl.find_matching_line(item)
            if not passed:
                if match:
                    res = "BFAIL"
                else:
                    res = "FAIL!"
            else:
                if match:
                    res = "BPASS"
                else:
                    res = "PASS"
            if item.fatal:
                # PYSIDE-1229: Stop the testing completely when a fatal error exists
                res = "FATAL"
            item.rich_result = res
            yield item

    def get_slowest_tests(self, max_count):
        result = self.results.copy()
        result.sort(key=sort_time_key, reverse=True)
        if len(result) > max_count:
            result = result[0:max_count - 1]
        return result



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' Test case for a class that holds a unknown handle object.
    Test case for BUG #1105.
'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import HandleHolder


class HandleHolderTest(unittest.TestCase):
    def testCreation(self):
        holder = HandleHolder(HandleHolder.createHandle())
        holder2 = HandleHolder(HandleHolder.createHandle())
        self.assertEqual(holder.compare(holder2), False)

    def testTransfer(self):
        holder = HandleHolder()
        holder2 = HandleHolder(holder.handle())
        self.assertTrue(holder.compare(holder2))

    def testUseDefinedType(self):
        holder = HandleHolder(8)
        holder2 = HandleHolder(holder.handle2())
        self.assertTrue(holder.compare2(holder2))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtWidgets import QMenu
from PySide6.QtCore import Signal, Slot


class RecentFileMenu(QMenu):
    fileOpened = Signal(str)

    def __init__(self, parent, recent):
        super().__init__(parent)
        self._recentFiles = recent
        self._recentFiles.changed.connect(self.updateList)
        self._recentFiles.destroyed.connect(self.deleteLater)
        self.updateList()

    @Slot()
    def updateList(self):
        for a in self.actions():
            del a

        if not self._recentFiles:
            self.addAction("<no recent files>")
            return

        for fileName in self._recentFiles.recentFiles():
            action = self.addAction(fileName)
            action.triggered.connect(self._emitFileOpened)

    @Slot()
    def _emitFileOpened(self):
        action = self.sender()
        self.fileOpened.emit(action.text())



# Copyright (C) 2026 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

from PySide6.QtCore import QObject, QMarginsF, QUrl, Qt, Property, Slot
from PySide6.QtGui import (QDesktopServices, QImage, QPageSize, QPainter, QPaintDevice,
                           QPdfWriter)
from PySide6.QtQml import QmlElement
from PySide6.QtPrintSupport import QPrinter, QPrinterInfo

QML_IMPORT_NAME = "GraphPrintingExample"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class GraphPrinter(QObject):

    def __init__(self, parent=None):
        super().__init__(parent)

    @Property(int, constant=True)
    def maxTextureSize(self):
        return 4096  # Use 4096 as the minimum

    def paintImage(self, image: QImage, device: QPaintDevice):
        painter = QPainter(device)
        viewportSize = painter.viewport().size()
        imageSize = image.size()
        print(f"Scaling {imageSize.width()}x{imageSize.height()} to "
              f"{viewportSize.width()}x{viewportSize.height()}.")
        finalImage = image.scaled(viewportSize, Qt.AspectRatioMode.KeepAspectRatio)
        painter.setRenderHint(QPainter.RenderHint.LosslessImageRendering)
        painter.drawImage(finalImage.rect(), finalImage)
        painter.end()

    def _generatePDF(self, fileName: str, image: QImage):
        writer = QPdfWriter(fileName)
        writer.setResolution(90)
        writer.setTitle("Graph")
        writer.setPageSize(QPageSize(image.size()))
        writer.setPageMargins(QMarginsF(0, 0, 0, 0))
        writer.newPage()
        self.paintImage(image, writer)

    @Slot(QUrl, QImage, result=str)
    def generatePDF(self, path: QUrl, image: QImage):
        fileName = path.toLocalFile()
        self._generatePDF(fileName, image)

        QDesktopServices.openUrl(path)

        return fileName

    @Slot(QImage, str, result=str)
    def print(self, image: QImage, printerName: str):
        printInfo = QPrinterInfo.printerInfo(printerName)
        if printInfo.isNull():
            return f"{printerName} is not a valid printer"

        printer = QPrinter(printInfo, QPrinter.PrinterMode.HighResolution)
        printer.setOutputFormat(QPrinter.OutputFormat.NativeFormat)
        self.paintImage(image, printer)

        return f"Printed to {printerName}"

    @Slot(result="QStringList")
    def getPrinters(self):
        return QPrinterInfo.availablePrinterNames()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test case for QQmlApplicationEngine'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.timedqguiapplication import TimedQGuiApplication

from PySide6.QtCore import QTimer
from PySide6.QtQml import QQmlApplicationEngine


class TestQQmlApplicationEngine(TimedQGuiApplication):

    def testQQmlApplicationEngine(self):
        engine = QQmlApplicationEngine()

        qml_file_path = Path(__file__).resolve().parent / "qqmlapplicationengine.qml"

        # PYSIDE-1736: load from a string.
        engine.load(os.fspath(qml_file_path))
        rootObjects = engine.rootObjects()
        self.assertTrue(rootObjects)
        window = rootObjects[0]
        window.setTitle("TestQQmlApplicationEngine")
        QTimer.singleShot(100, window.close)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for PYSIDE-344, imul/idiv are used instead of mul/div,
   modifying the argument passed in'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QMargins, QPoint, QPointF, QSize, QSizeF
from PySide6.QtGui import (QMatrix4x4, QQuaternion, QTransform, QVector2D,
                           QVector3D, QVector4D)


def testList():
    return [QPoint(10, 10), QPointF(1, 1), QSize(10, 10), QSizeF(1, 1),
            QMargins(10, 10, 10, 10),
            QTransform(), QMatrix4x4(),
            QVector2D(1, 1), QVector3D(1, 1, 1), QVector4D(1, 1, 1, 1),
            QQuaternion(1, 1, 1, 1)]


class TestMulDiv(unittest.TestCase):

    def testMultiplication(self):
        fails = ''
        for a in testList():
            mul = (a * 2)
        if a == mul:
            fails += ' ' + type(a).__name__
        self.assertEqual(fails, '')

    def testDivision(self):
        fails = ''
        for a in testList():
            div = (a * 2)
            if a == div:
                fails += ' ' + type(a).__name__
        self.assertEqual(fails, '')


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QtAsyncio'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

import asyncio
import datetime
from PySide6.QtAsyncio import QAsyncioEventLoopPolicy


class QAsyncioTestCaseTime(unittest.TestCase):

    def setUp(self):
        self.previous_time = None
        self.exception = None

    def display_date(self, end_time, loop):
        if self.previous_time is not None:
            try:
                self.assertAlmostEqual(
                    (datetime.datetime.now() - self.previous_time).total_seconds(), 1, delta=0.1)
            except AssertionError as e:
                self.exception = e
        self.previous_time = datetime.datetime.now()
        if (loop.time() + 1.0) < end_time:
            loop.call_later(1, self.display_date, end_time, loop)
        else:
            loop.stop()

    def test_time(self):
        asyncio.set_event_loop_policy(QAsyncioEventLoopPolicy())
        loop = asyncio.new_event_loop()

        end_time = loop.time() + 3.0
        loop.call_soon(self.display_date, end_time, loop)

        try:
            loop.run_forever()
        finally:
            loop.close()

        if self.exception is not None:
            raise self.exception


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
from sample import ByteArray


class ByteArrayBufferProtocolTest(unittest.TestCase):
    '''Tests ByteArray implementation of Python buffer protocol.'''

    def testByteArrayBufferProtocol(self):
        # Tests ByteArray implementation of Python buffer protocol using the Path().is_dir
        # function with a unicode object or other object implementing the Python buffer protocol.
        Path(str(ByteArray('/tmp'))).is_dir()


class ByteArrayConcatenationOperatorTest(unittest.TestCase):
    '''Test cases for ByteArray concatenation with '+' operator.'''

    def testConcatByteArrayAndPythonString(self):
        # Test concatenation of a ByteArray with a Python string, in this order.
        ba = ByteArray('foo')
        result = ba + '\x00bar'
        self.assertEqual(type(result), ByteArray)
        self.assertEqual(result, 'foo\x00bar')

    def testConcatPythonStringAndByteArray(self):
        # Test concatenation of a Python string with a ByteArray, in this order.
        concat_python_string_add_qbytearray_worked = True  # noqa: F841
        ba = ByteArray('foo')
        result = 'bar\x00' + ba
        self.assertEqual(type(result), ByteArray)
        self.assertEqual(result, 'bar\x00foo')


class ByteArrayOperatorEqual(unittest.TestCase):
    '''TestCase for operator ByteArray == ByteArray.'''

    def testDefault(self):
        # ByteArray() == ByteArray()
        obj1 = ByteArray()
        obj2 = ByteArray()
        self.assertEqual(obj1, obj2)

    def testSimple(self):
        # ByteArray(some_string) == ByteArray(some_string)
        string = 'egg snakes'
        self.assertEqual(ByteArray(string), ByteArray(string))

    def testPyString(self):
        # ByteArray(string) == string
        string = 'my test string'
        self.assertEqual(ByteArray(string), string)

    def testQString(self):
        # ByteArray(string) == string
        string = 'another test string'
        self.assertEqual(ByteArray(string), string)


class ByteArrayOperatorAt(unittest.TestCase):
    '''TestCase for operator ByteArray[]'''

    def testInRange(self):
        # ByteArray[x] where x is a valid index.
        string = 'abcdefgh'
        obj = ByteArray(string)
        for i in range(len(string)):
            self.assertEqual(obj[i], bytes(string[i], "UTF8"))

    def testInRangeReverse(self):
        # ByteArray[x] where x is a valid index (reverse order).
        string = 'abcdefgh'
        obj = ByteArray(string)
        for i in range(len(string) - 1, 0, -1):
            self.assertEqual(obj[i], bytes(string[i], "UTF8"))

    def testOutOfRange(self):
        # ByteArray[x] where x is out of index.
        string = '1234567'
        obj = ByteArray(string)
        self.assertRaises(IndexError, lambda: obj[len(string)])

    def testNullStrings(self):
        ba = ByteArray('\x00')
        self.assertEqual(ba.at(0), '\x00')
        self.assertEqual(ba[0], bytes('\x00', "UTF8"))


class ByteArrayOperatorLen(unittest.TestCase):
    '''Test case for __len__ operator of ByteArray'''

    def testBasic(self):
        '''ByteArray __len__'''
        self.assertEqual(len(ByteArray()), 0)
        self.assertEqual(len(ByteArray('')), 0)
        self.assertEqual(len(ByteArray(' ')), 1)
        self.assertEqual(len(ByteArray('yabadaba')), 8)


class ByteArrayAndPythonStr(unittest.TestCase):
    '''Test case for __str__ operator of ByteArray'''

    def testStrOperator(self):
        '''ByteArray __str__'''
        self.assertEqual(ByteArray().__str__(), '')
        self.assertEqual(ByteArray('').__str__(), '')
        self.assertEqual(ByteArray('aaa').__str__(), 'aaa')

    def testPythonStrAndNull(self):
        s1 = bytes('123\000321', "UTF8")
        ba = ByteArray(s1)
        s2 = ba.data()
        self.assertEqual(s1, s2)
        self.assertEqual(type(s1), type(s2))
        self.assertEqual(s1, ba)
        self.assertNotEqual(type(s1), type(ba))


if __name__ == '__main__':
    unittest.main()
